/*
 *  Implementation of COSS Naming Service for MICO
 *  Copyright (C) 1997 Kai-Uwe Sattler
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef Naming_impl_h_
#define Naming_impl_h_

#include <naming.h>
#include <mico/template_impl.h>

struct NmCtxBinding {
  CosNaming::BindingType btype;
  CosNaming::NameComponent name;
  CORBA::Object_ptr obj;
};

class NamingContext_impl : virtual public CosNaming::NamingContext_skel 
{
private:
  struct str_less : binary_function<CosNaming::Istring_var, 
    CosNaming::Istring_var, bool> {
      bool operator() (const CosNaming::Istring_var& s1, 
		      const CosNaming::Istring_var& s2) const {
	return (strcmp ((const char *)s1, (const char *) s2) < 0);
      }
  };

  typedef map<CosNaming::Istring_var, NmCtxBinding, str_less> BindingTable;

  BindingTable table;

protected:
  NamingContext_impl (const CORBA::BOA::ReferenceData &);

  void _do_bind (const CosNaming::Name& n, CORBA::Object_ptr obj,
		 CosNaming::BindingType btype, CORBA::Boolean rebind);

public:
  NamingContext_impl (CORBA::Object_ptr obj);

  static CosNaming::NamingContext_ptr _create_context (const char * = "");

  void bind (const CosNaming::Name& n, CORBA::Object_ptr obj);
  void rebind (const CosNaming::Name& n, CORBA::Object_ptr obj);
  void bind_context (const CosNaming::Name& n, 
		     CosNaming::NamingContext_ptr nc);
  void rebind_context (const CosNaming::Name& n, 
		       CosNaming::NamingContext_ptr nc);
  CORBA::Object_ptr resolve (const CosNaming::Name& n);
  void unbind (const CosNaming::Name& n);
  CosNaming::NamingContext_ptr new_context ();
  CosNaming::NamingContext_ptr bind_new_context (const CosNaming::Name& n);
  void destroy ();
  void list (CORBA::ULong how_many, CosNaming::BindingList*& bl, 
	     CosNaming::BindingIterator_ptr& bi);

  CORBA::Boolean _save_object ();

  static void _shutdown ();

private:
  static NamingContext_impl* root_impl;
};

class BindingIterator_impl : virtual public CosNaming::BindingIterator_skel
{
private:
  CORBA::ULong idx;
  CosNaming::BindingList blist;

public:
  BindingIterator_impl (const CosNaming::BindingList& bl);

  CORBA::Boolean next_one (CosNaming::Binding*& b);
  CORBA::Boolean next_n (CORBA::ULong how_many, CosNaming::BindingList*& bl);
  void destroy ();
};

class NamingContextLoader : public CORBA::BOAObjectRestorer 
{
public:
  CORBA::Boolean restore (CORBA::Object_ptr obj);
};

#endif
