/*
 *  MICO --- a free CORBA implementation
 *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#include <CORBA.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <iostream.h>
#include <fstream.h>
#include <mico/template_impl.h>
#include <mico/util.h>
#include "Naming_impl.h"

void sighandler (int sig)
{
  NamingContext_impl::_shutdown ();
}

void usage (const char *progname)
{
  cerr << "usage: " << progname << " [<options>]" << endl;
  cerr << "possible <options> are:" << endl;
  exit (1);
}

int main (int argc, char *_argv[])
{
  signal (SIGINT, sighandler);
#if 0
  /*
   * when exiting micod sends all running servers a SIGTERM. that is
   * we can't save on SIGTERM ...
   */
  signal (SIGTERM, sighandler);
#endif
  // create loader for persistent state 
  NamingContextLoader loader;

  // prevent a bind() to naming service...
  char **argv = new char*[argc+2+1];
  for (int i=0; i<argc; ++i)
      argv[i] = _argv[i];
  argv[argc]   = "-ORBNamingAddr";
  argv[argc+1] = "";
  argv[argc+2] = 0;
  argc += 2;

  // ORB initialization
  CORBA::ORB_var orb = CORBA::ORB_init (argc, argv, "mico-local-orb");
  CORBA::BOA_var boa = orb->BOA_init (argc, argv, "mico-local-boa");

  if (!boa->restoring ()) {
    CosNaming::NamingContext_ptr nctx =
	NamingContext_impl::_create_context ("root");
    CORBA::String_var ref = orb->object_to_string (nctx);
//    cout << "NamingContext = " << ref << endl;
  }

  boa->impl_is_ready (CORBA::ImplementationDef::_nil ());
  orb->run ();

  return 0;
}
