/*

    Copyright (C) 1998 Stefan Westerfeld
                       stefan@space.twc.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Adapted by Arno Puder.
*/


#include <CORBA.h>
#include <fstream.h>
#include <ctype.h>
#include <stdio.h>
#include "codegen-c++-impl.h"
#include <mico/template_impl.h>
#include <mico/util.h>

#define _IMPL "_impl"


CodeGenCPPImpl::CodeGenCPPImpl( DB &db, IDLParam &params )
  : CodeGen( db.get_idl_container() ),
    CodeGenCPPUtil( db, params )
{
}



void CodeGenCPPImpl::emit_impl_h()
{
  headers = true;
  emit_impl();
}

void CodeGenCPPImpl::emit_impl_cpp()
{
  headers = false;
  emit_impl();
}

void CodeGenCPPImpl::emit_impl()
{
  for( CORBA::ULong i = 0; i < _idl_objs.length(); i++ ) {
    
    CORBA::InterfaceDef_var in;
    in = CORBA::InterfaceDef::_narrow( _idl_objs[ i ] );
    if( CORBA::is_nil( in ) )
      continue;
    
    if( check_for_included_defn( in ) )
      continue;
    
    emit_Interface( in );
  }
}

void CodeGenCPPImpl::emit_your_impl_here()
{
  // implementation not in header ...
  if(headers)
  {
    o << ";" << endl;
  }
  else
  {
    o << endl << BL_OPEN;
    o << "// add your implementation here" << endl;
    o << BL_CLOSE << endl;
  }
}

void CodeGenCPPImpl::emitPrototypes( CORBA::InterfaceDef_ptr in,
				     bool as_pure_virtual )
{
  CORBA::InterfaceDef::FullInterfaceDescription_var desc;
  desc = in->describe_interface();

  CORBA::String_var tmp;
  tmp = in->name(); //in->absolute_name();
  string implClassName="";
  if(!headers)
  {
    implClassName = ID(tmp);
    implClassName += _IMPL;
    implClassName += "::";
  }

  // Emit attributes
  for( CORBA::ULong j = 0; j < desc->attributes.length(); j++ ) {
    CORBA::AttributeDescription attr;
    attr = desc->attributes[ j ];
    if( as_pure_virtual )
      o << "virtual ";
    CORBA::IDLType_var type_id = lookup_attribute_by_id( attr.id );
    emit_type_for_result( type_id );
    o << " " << implClassName << ID( attr.name ) << "()";
    if( as_pure_virtual )
      o << " = 0";
    emit_your_impl_here();

    if( attr.mode == CORBA::ATTR_NORMAL ) {
      if( as_pure_virtual )
	o << "virtual ";
      o << "void " << implClassName << ID( attr.name ) << "( ";
      emit_type_for_param( type_id, CORBA::PARAM_IN );
      o << " value )";
      if( as_pure_virtual )
	o << " = 0";
      emit_your_impl_here();
    }
  }

  // Emit operations
  for( CORBA::ULong j0 = 0; j0 < desc->operations.length(); j0++ ) {
    CORBA::OperationDescription op;
    op = desc->operations[ j0 ];
    if( as_pure_virtual )
      o << "virtual ";
    CORBA::IDLType_var type_id = lookup_result_by_id( op.id );
    emit_type_for_result( type_id );
    o << " " <<implClassName << ID(op.name) << "(";
    CORBA::ParDescriptionSeq p = op.parameters;
    CORBA::ULong k;
    for( k = 0; k < p.length(); k++ ) {
      if( k != 0 )
	o << ",";
      o << " ";
      emit_type_for_param( p[ k ].type_def, p[ k ].mode );
      o << " " << ID(p[ k ].name);
    }

    // Do we have a context?
    CORBA::ContextIdSeq& ctx = op.contexts;
    
    if( ctx.length() > 0 ) {
      if( p.length() > 0 )
	o << ", ";
      o << "CORBA::Context_ptr _ctx";
    }
    
    if( k != 0 || ctx.length() > 0 )
      o << " ";
    o << ")";
    if( as_pure_virtual )
      o << " = 0";
    emit_your_impl_here();
  }
}

void CodeGenCPPImpl::emit_Interface( CORBA::InterfaceDef_ptr in )
{
  assert( in->def_kind() == CORBA::dk_Interface );

  set_name_prefix( in );
  
  CORBA::String_var n = in->name();
  string className = (const char *) ID(n);
  string skelName = (const char *) ID(in->absolute_name());

  skelName = skelName.substr(2) + _SKEL;
 
  o << endl;

  //-----IMPL--------------------------------------------------------
  o << "// Implementation for interface " << className << endl;

  if(headers)
  {
    // note: our skeleton must be the rightmost entry in the inheritance list!
    o << "class " << className << _IMPL << " : ";
  
    CORBA::InterfaceDefSeq_var bases = in->base_interfaces();
    if( bases->length() == 0 ) {
      // This interface has no base interfaces
      o << "virtual public " << skelName;
      o << endl;
    } else {
      // This interface *has* base interfaces
      o << indent << endl;
      for( CORBA::ULong i = 0; i < bases->length(); i++ ) {
        o << "virtual public ";
        CORBA::String_var base_name = bases[ i ]->name();
        o << ID(base_name);
        o << _IMPL;
	o << ",";
        o << endl;
      }
      o << "virtual public " << skelName << endl;
      o << exdent;
    }
  
    o << BL_OPEN;
  
    o << "public:" << indent << endl;
  }
  
  // Generate prototypes
  emitPrototypes( in, false );
 
  if(headers)
  { 
    // Generate epilogue
    o << exdent << BL_CLOSE_SEMI << endl;
  }
}
