/*
 *  MICO --- a free CORBA implementation
 *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */


#include <CORBA.h>
#include <fstream.h>
#include <ctype.h>
#include <stdio.h>
#include "codegen-c++-skel.h"
#include <mico/template_impl.h>
#include <mico/util.h>


CodeGenCPPSkel::CodeGenCPPSkel( DB &db, IDLParam &params )
  : CodeGen( db.get_idl_container() ),
    CodeGenCPPUtil( db, params )
{
}



void CodeGenCPPSkel::emit_skel()
{
  CORBA::InterfaceDef::FullInterfaceDescription_var desc;
  CORBA::IDLType_var idl_type;
  CORBA::String_var tmp;
  
  for( CORBA::ULong i0 = 0; i0 < _idl_objs.length(); i0++ ) {

    CORBA::InterfaceDef_var in;
    in = CORBA::InterfaceDef::_narrow( _idl_objs[ i0 ] );
    if( CORBA::is_nil( in ) )
      continue;
    
    if( check_for_included_defn( in ) )
      continue;
    
    set_name_prefix( in );
    tmp = in->name();
    string className;
    className = ID(tmp);
    tmp = in->absolute_name();
    string absClassName;
    absClassName = ID(tmp);
    absClassName = absClassName.substr( 2 );
    CORBA::String_var in_id = in->id();
    desc = in->describe_interface();

    if (_params.boa) {
      o << endl;
      o << "// Dynamic Implementation Routine for interface " <<
	className << endl;
      o << absClassName << _SKEL << "::" << className << _SKEL;
      o << "( const CORBA::BOA::ReferenceData &_id )" << endl;
      o << BL_OPEN;
      o << "CORBA::ImplementationDef_var _impl =" << indent << endl;
      o << "_find_impl( \"" << in_id << "\", \"";
      o << className << "\" );" << exdent << endl;
      o << "assert( !CORBA::is_nil( _impl ) );" << endl;
      o << "_create_ref( _id," << indent << endl;
      o << "CORBA::InterfaceDef::_nil()," << endl;
      o << "_impl," << endl;
      o << "\"" << in_id << "\" );" << exdent << endl;
      o << "register_dispatcher( new InterfaceDispatcherWrapper<";
      o << className << _SKEL;
      o << ">( this ) );" << endl;
      o << BL_CLOSE << endl;
      
      o << absClassName << _SKEL << "::" << className << _SKEL;
      o << "( CORBA::Object_ptr _obj )" << endl;
      o << BL_OPEN;
      o << "CORBA::ImplementationDef_var _impl =" << indent << endl;
      o << "_find_impl( \"" << in->id() << "\", \"";
      o << className << "\" );" << exdent << endl;
      o << "assert( !CORBA::is_nil( _impl ) );" << endl;
      o << "_restore_ref( _obj," << indent << endl;
      o << "CORBA::BOA::ReferenceData()," << endl;
      o << "CORBA::InterfaceDef::_nil()," << endl;
      o << "_impl );" << exdent << endl;
      o << "register_dispatcher( new InterfaceDispatcherWrapper<";
      o << className << _SKEL;
      o << ">( this ) );" << endl;
      o << BL_CLOSE << endl;

      // dtor
      o << absClassName << _SKEL;
      o << "::~" << className << _SKEL << "()" << endl;
      o << "{" << endl;
      o << "}" << endl << endl;

      // Generate method dispatcher
      o << "bool " << absClassName
	<< _SKEL << "::dispatch( CORBA::ServerRequest_ptr ";

      if (desc->attributes.length() + desc->operations.length() == 0) {
	o << "/*_req*/";
      } else {
	o << "_req";
      }
      o << ", CORBA::Environment & /*_env*/ )"
	<< endl << BL_OPEN;
      
      // Generate dispatcher for attributes
      for( CORBA::ULong j0 = 0; j0 < desc->attributes.length(); j0++ ) {
	emit_attr_dispatcher (desc->attributes[j0]);
      }
      
      // Generate dispatcher for methods
      for( CORBA::ULong j1 = 0; j1 < desc->operations.length(); j1++ ) {
	CORBA::OperationDescription op;
	op = desc->operations[ j1 ];
	string dispatch_name = (const char *) op.name;
	idl_type = lookup_result_by_id( op.id );
	emit_method_dispatcher( dispatch_name, op, idl_type );
      }
      o << "return false;" << endl;
      o << BL_CLOSE << endl;
      
      // _this method
      o << absClassName << "_ptr " << absClassName << _SKEL;
      o << "::_this()" << endl;
      o << BL_OPEN;
      o << "return " << absClassName << "::_duplicate( this );" << endl;
      o << BL_CLOSE << endl;
    }
      
    /*
     * PortableServer Skeleton class
     */

    if (_params.poa) {
      string prefix;
      CORBA::Container_var def_in = in->defined_in ();
      if (def_in->def_kind() == CORBA::dk_Repository) {
	prefix = (const char *) "POA_";
      }
      else {
	prefix = (const char *) "";
      }

      bool oldurn = use_rel_names (false);
      
      o << endl;
      o << "// PortableServer Skeleton Class for interface " 
	<< absClassName << endl;
      
      /*
       * virtual destructor
       */
      o << "POA_" << absClassName << "::~" << prefix << className << "()"
	<< endl;
      o << BL_OPEN << BL_CLOSE << endl;

      /*
       * _this() method
       */
      o << absClassName << "_ptr" << endl;
      o << "POA_" << absClassName << "::_this ()" << endl;
      o << BL_OPEN;
      o << "CORBA::Object_var obj = PortableServer::DynamicImplementation::_this ();" << endl;
      o << "return " << absClassName << "::_narrow (obj);" << endl;
      o << BL_CLOSE << endl;

      /*
       * _is_a() method
       */
      o << "bool" << endl;
      o << "POA_" << absClassName << "::_is_a (const char * repoid)" << endl;
      o << BL_OPEN;
      o << "if (strcmp (repoid, \"" << in_id << "\") == 0) " << BL_OPEN;
      o << "return true;" << endl;
      o << BL_CLOSE;
      CORBA::InterfaceDefSeq_var bases = in->base_interfaces();
      for (CORBA::ULong j2=0; j2<bases->length(); j2++) {
	tmp = bases[j2]->absolute_name();
	string base_name;
	base_name = ID(tmp);
	base_name = base_name.substr( 2 );
	o << "if (POA_" << base_name << "::_is_a (repoid)) " << BL_OPEN;
	o << "return true;" << endl;
	o << BL_CLOSE;
      }
      o << "return false;" << endl;
      o << BL_CLOSE << endl;

      /*
       * _primary_interface() method
       */
      o << "CORBA::RepositoryId" << endl;
      o << "POA_" << absClassName << "::_primary_interface";
      o <<   " (const PortableServer::ObjectId & id,";
      o <<    " PortableServer::POA_ptr poa)" << endl;
      o << BL_OPEN;
      o << "return CORBA::string_dup (\"" << in_id << "\");" << endl;
      o << BL_CLOSE << endl;
      
      /*
       * method dispatcher
       */
      o << "bool" << endl;
      o << "POA_" << absClassName << "::dispatch (CORBA::ServerRequest_ptr _req)"
	<< endl;
      o << BL_OPEN;
      
      // Generate dispatcher for attributes
      for( CORBA::ULong j3 = 0; j3 < desc->attributes.length(); j3++ ) {
	emit_attr_dispatcher (desc->attributes[j3]);
      }
      
      // Generate dispatcher for methods
      for( CORBA::ULong j4 = 0; j4 < desc->operations.length(); j4++ ) {
	CORBA::OperationDescription op;
	op = desc->operations[ j4 ];
	string dispatch_name = (const char *) op.name;
	idl_type = lookup_result_by_id( op.id );
	emit_method_dispatcher( dispatch_name, op, idl_type );
      }
      // call base classes' dispatchers
      if (bases->length()) {
	o << endl;
      }
      for (CORBA::ULong j5=0; j5<bases->length(); j5++) {
	tmp = bases[j5]->absolute_name();
	string base_name;
	base_name = ID(tmp);
	base_name = base_name.substr( 2 );
	o << "if (POA_" << base_name << "::dispatch (_req)) " << BL_OPEN;
	o << "return true;" << endl;
	o << BL_CLOSE << endl;
      }
      o << "return false;" << endl;
      o << BL_CLOSE << endl;
      
      // invoke() method
      o << "void" << endl;
      o << "POA_" << absClassName << "::invoke (CORBA::ServerRequest_ptr _req)"
	<< endl;
      o << BL_OPEN;
      o << "if (dispatch (_req)) " << BL_OPEN;
      o << "  return;" << endl;
      o << BL_CLOSE;
      o << endl;
      o << "CORBA::Exception * ex = " << endl;
      o << "  new CORBA::BAD_OPERATION (0, CORBA::COMPLETED_NO);" << endl;
      o << "_req->exception (ex);" << endl;
      o << BL_CLOSE << endl;
      use_rel_names (oldurn);
    }
  }
}

void
CodeGenCPPSkel::emit_attr_dispatcher (CORBA::AttributeDescription &attr)
{
  CORBA::OperationDescription op;
  CORBA::IDLType_var idl_type;

  op.name = (const char *) attr.name;
  op.id = (const char *) "";  // Not needed
  op.defined_in = attr.defined_in;
  op.version = (const char*) attr.version;
  op.result = CORBA::TypeCode::_nil();  // Not needed
  op.mode = CORBA::OP_NORMAL;
  // op.contexts not needed for attributes
  op.parameters.length( 0 );  // No parameters for _get
  // op.exceptions not needed for attributes

  string dispatch_name = string("_get_") + (const char *)attr.name;
    
  idl_type = lookup_attribute_by_id( attr.id );
  emit_method_dispatcher( dispatch_name, op, idl_type );

  if( attr.mode == CORBA::ATTR_NORMAL ) {
    // We also need the set version of the attribute
    CORBA::ParameterDescription attr_param;
    attr_param.name = (const char *) "_value";
    attr_param.type = attr.type;
    attr_param.type_def = idl_type;
    attr_param.mode = CORBA::PARAM_IN;
    
    op.name = (const char *) attr.name;
    op.id = (const char *) "";  // Not needed
    op.defined_in = attr.defined_in;
    op.version = (const char*) attr.version;
    op.result = CORBA::TypeCode::_nil();  // Not needed
    op.mode = CORBA::OP_NORMAL;
    // op.contexts not needed for attributes
    op.parameters.length( 1 );
    op.parameters[ 0 ] = attr_param;
    // op.exceptions not needed for attributes
    
    dispatch_name = string("_set_") + (const char *)attr.name;
    
    emit_method_dispatcher( dispatch_name, op,
			    _repo->get_primitive( CORBA::pk_void ) );
  }
}

void CodeGenCPPSkel::emit_method_dispatcher( string &dispatch_name,
					     CORBA::OperationDescription &op,
					     CORBA::IDLType_ptr result )
{
  o << "if( strcmp( _req->op_name(), \"" << 
    ID( (char *) dispatch_name.c_str() ) << "\" ) == 0 ) " << BL_OPEN;
  CORBA::ParDescriptionSeq &p = op.parameters;
  CORBA::ULong k;

  // Emit variable declarations for all parameters as well as
  // the wrappers for the array parameters
  for( k = 0; k != p.length(); k++ ) {
    // Declaration for parameter
    if( p[ k ].mode != CORBA::PARAM_OUT ) {
      emit_type_for_variable( p[ k ].type_def, automatic );
      o << " " << ID(p[ k ].name);
      emit_array_suffix( p[ k ].type_def );
      o << ";" << endl;
    } else {
      if( is_array( p[ k ].type ) && is_variable( p[ k ].type ) ) {
	emit_type_for_variable( p[ k ].type_def, automatic );
	o << "_slice*";
      } else
	emit_type_for_variable( p[ k ].type_def, manual );
      o << " " << ID(p[ k ].name) << ";" << endl;
    }
    // Wrapper for array parameter
    if( is_array( p[ k ].type ) ) {
      if( !is_variable( p[ k ].type ) || p[ k ].mode != CORBA::PARAM_OUT ) {
	emit_type_for_variable( p[ k ].type_def, automatic );
	o << "_forany _forany_" << ID(p[ k ].name);
	o << "( " << ID(p[ k ].name) << " );" << endl;
      }
    }
  }
  if( k != 0 )
    o << endl;
  
  o << "CORBA::NVList_ptr _args = new CORBA::NVList (" 
    << p.length() << ");" << endl;
  
  for( k = 0; k != p.length(); k++ ) {
    o << "_args->add( CORBA::";
    switch( p[ k ].mode ) {
    case CORBA::PARAM_IN:
      o << "ARG_IN";
      break;
    case CORBA::PARAM_OUT:
      o << "ARG_OUT";
      break;
    case CORBA::PARAM_INOUT:
      o << "ARG_INOUT";
      break;
    default:
      assert( 0 );
    }
    o << " );" << endl;
    o << "_args->item( " << k << " )->value()->type( ";
    emit_type_code_name( p[ k ].type_def );
    o << " );" << endl;
  }
  o << endl;
  
  o << "if (!_req->params( _args ))" << endl;
  o << indent << "return true;" << exdent << endl << endl;
  
  for( k = 0; k < p.length(); k++ ) {
    if( p[ k ].mode != CORBA::PARAM_OUT ) {
      o << "*_args->item( " << k << " )->value() >>= ";
      if( is_string( p[ k ].type ) ) {
	o << "CORBA::Any::to_string( " << ID(p[ k ].name) << ", ";
	CORBA::TypeCode_var tc_string = resolve_alias( p[ k ].type );
	o << tc_string->length() << " )";
      } else if( is_array( p[ k ].type ) ) {
	o << "_forany_" << ID(p[ k ].name);
      } else {
	emit_any_param( p[ k ].type_def, p[ k ].name, false );
      }
      o << ";" << endl;
    }
  }

  // Do we have a result for this operation?
  CORBA::TypeCode_var tc_result = result->type();
  if( tc_result->kind() != CORBA::tk_void ) {
    if( is_array( tc_result ) ) {
      emit_type_for_variable( result, automatic );
      o << "_slice*";
    } else
      emit_type_for_variable( result, manual );
    o << " _res;" << endl;
  }

  if( !_params.no_exceptions && op.mode == CORBA::OP_NORMAL ) {
    o << "#ifdef HAVE_EXCEPTIONS" << endl;
    o << "try {" << endl;
    o << "#endif" << indent << endl;
  }
  
  // Do we have a result for this operation?
  if( tc_result->kind() != CORBA::tk_void ) {
    o << "_res = ";
  }

  // Call the implementation
  o << ID(op.name) << "(";
  for( k = 0; k < p.length(); k++ ) {
    if( k != 0 )
      o << ",";
    o << " " << ID(p[ k ].name);
  }
  CORBA::ContextIdSeq& ctx = op.contexts;
  if( ctx.length() > 0 ) {
    if( p.length() > 0 )
      o << ", ";
    o << "_req->ctx()";
  }
  if( k != 0 || ctx.length() > 0 )
    o << " ";
  o << ");" << endl;

  if( !_params.no_exceptions && op.mode == CORBA::OP_NORMAL ) {
    o << exdent << "#ifdef HAVE_EXCEPTIONS" << endl;
    CORBA::ExcDescriptionSeq exs = op.exceptions;
    for( CORBA::ULong i = 0; i < exs.length(); i++ ) {
      CORBA::Contained_var con = _repo->lookup_id( exs[ i ].id );
      assert( !CORBA::is_nil( con ) );
      CORBA::String_var tmp = con->absolute_name();
      CORBA::String_var ex_abs_name = (const char *) ID(tmp);
      o << "} catch( " << ex_abs_name << "_var &_ex ) " << BL_OPEN;
      o << "_req->exception( _ex->_clone() );" << endl;
      o << "return true;" << exdent << endl;
    }
    o << "} catch( CORBA::SystemException_var &_ex ) " << BL_OPEN;
    o << "_req->exception( _ex->_clone() );" << endl;
    o << "return true;" << exdent << endl;
    o << "} catch( ... ) " << BL_OPEN;
    o << "assert( 0 );" << endl;
    o << "return true;" << endl;
    o << BL_CLOSE;
    o << "#endif" << endl;
  }
  
  // Write back OUT and INOUT parameters
  for( k = 0; k < p.length(); k++ ) {
    if( p[ k ].mode != CORBA::PARAM_IN ) {
      // Do we need a wrapper for an out variable length array?
      if( is_variable( p[ k ].type ) && is_array( p[ k ].type ) &&
	  p[ k ].mode == CORBA::PARAM_OUT ) {
	// Yes
	emit_type_for_variable( p[ k ].type_def, automatic );
	o << "_forany _forany_" << ID(p[ k ].name);
	o << "( " << ID(p[ k ].name) << " );" << endl;
      }
      
      o << "*_args->item( " << k << " )->value() <<= ";
      if( is_string( p[ k ].type ) ) {
	o << "CORBA::Any::from_string( (char *) " << ID(p[ k ].name) << ", ";
	CORBA::TypeCode_var tc_string = resolve_alias( p[ k ].type );
	o << tc_string->length() << " );" << endl;
	if( p[ k ].mode == CORBA::PARAM_OUT )
	  o << "CORBA::string_free( " << ID(p[ k ].name) << " );" << endl;
      } else if( is_array( p[ k ].type ) ) {
	o << "_forany_" << ID(p[ k ].name) << ";" << endl;
      } else {
	string n;
	if( is_variable( p[ k ].type ) && !is_objref( p[ k ].type ) &&
	    p[ k ].mode == CORBA::PARAM_OUT )
	  n = "*";
	n += ID(p[ k ].name);
	emit_any_param( p[ k ].type_def, (char *) n.c_str(), true );
	o << ";" << endl;
	if( is_objref( p[ k ].type ) && p[ k ].mode == CORBA::PARAM_OUT )
	  o << "CORBA::release( " << ID(p[ k ].name) << " );" << endl;
	if( is_variable( p[ k ].type ) && !is_objref( p[ k ].type ) &&
	    p[ k ].mode == CORBA::PARAM_OUT )
	  o << "delete " << ID(p[ k ].name) << ";" << endl;
      }
    }
  }
  
  // Write back result (if there was one)
  if( tc_result->kind() != CORBA::tk_void ) {
    o << "CORBA::Any *_any_res = new CORBA::Any;" << endl;
    // Do we have to generate a wrapper for an array result?
    if( is_array( tc_result ) ) {
      // Yes
      emit_type_for_variable( result, automatic );
      o << "_forany _res_forany( _res );" << endl;
    }
    o << "*_any_res <<= ";
    if( is_array( tc_result ) ) {
      o << "_res_forany;" << endl;
      emit_idl_type_name( result );
      o << "_free( _res );" << endl;
    } else {
      if( !is_objref( tc_result ) && !is_string( tc_result ) &&
	  !is_wstring( tc_result ) && is_variable( tc_result ) ) {
	o << "*_res;" << endl;
	o << "delete _res;" << endl;
      } else {
	emit_any_param( result, "_res", true );
	o << ";" << endl;
	if( is_string( tc_result ) )
	  o << "CORBA::string_free( _res );" << endl;
	if( is_wstring( tc_result ) )
	  o << "CORBA::wstring_free( _res );" << endl;
	if( is_objref( tc_result ) )
	  o << "CORBA::release( _res );" << endl;
      }
    }
    o<< "_req->result( _any_res );" << endl;
  }

  o << "return true;" << endl;
  o << BL_CLOSE;
}

