/*
 *  MICO --- a free CORBA implementation
 *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#ifndef __CODEGENC_H__
#define __CODEGENC_H__

#include "idlparser.h"
#include "codegen.h"
#include "db.h"


#define _STUB   "_stub"
#define _SKEL   "_skel"



class CodeGenC : public CodeGen
{
private:
  DB*                         _db;
  set<string, less<string> >  _include_files;
  bool                        _gen_exceptions;
  bool                        _query_ir_for_narrow;
  string                      _cpp_suffix;
  string                      _h_suffix;
  
  enum Storage {
    manual,
    automatic
  };
  
  char* ID( char* ident );
  void emit_mangled_name( char *s );
  bool check_for_included_defn( CORBA::IRObject_ptr obj );
  
  void emitHeader();
  void emit_header_idl_obj( CORBA::IRObject_ptr obj );
  void emitHeaderStringDef( CORBA::StringDef_ptr s );
  void emitHeaderInterface( CORBA::InterfaceDef_ptr in );
  void emitHeaderDefaultMembers( char *name );
  void emitHeaderStruct( CORBA::StructDef_ptr s );
  void emitHeaderUnion( CORBA::UnionDef_ptr u );
  void emitHeaderConstant( CORBA::ConstantDef_ptr c );
  void emitHeaderException( CORBA::ExceptionDef_ptr e );
  void emitHeaderEnum( CORBA::EnumDef_ptr e );
  void emitHeaderAlias( CORBA::AliasDef_ptr a );

  void emitPrototypes( CORBA::InterfaceDef_ptr in,
		       bool as_pure_virtual );
  void emitForwardDcl();
  void emitLocalDecls( CORBA::InterfaceDef_ptr in );

  void emitAnyParam( CORBA::IDLType_ptr p,
		     char *var_name, bool is_from );

  void emit_array_marshalling( CORBA::ArrayDef_ptr arr,
			       bool is_from,
			       int level );
  void emit_stub_method( string &absClassName,
			 string &dispatch_name,
			 CORBA::OperationDescription &op,
			 CORBA::IDLType_ptr result );
  void emit_method_dispatcher( string &absClassName,
			       string &dispatch_name,
			       CORBA::OperationDescription &op,
			       CORBA::IDLType_ptr result );

  void emitClient();
  void emitClientStringDef( CORBA::StringDef_ptr s );
  void emitClientInterface( CORBA::InterfaceDef_ptr in );
  void emitClientEnum( CORBA::EnumDef_ptr s );
  void emitClientException( CORBA::ExceptionDef_ptr e );
  void emitClientStructDefaultMembers( char *name,
				       char *prefix,
				       CORBA::StructMemberSeq_var &s );
  void emitClientStruct( CORBA::StructDef_ptr s );
  void emitClientUnionDefaultMembers( char *name,
				      char *prefix,
				      CORBA::UnionMemberSeq_var &u );
  bool is_union_case_defined( CORBA::UnionMemberSeq& mem,
			      CORBA::Any &any );
  string generate_union_default_case_label( CORBA::IDLType_ptr discr,
					    CORBA::UnionMemberSeq &mem );
  void emit_union_case_label( CORBA::IDLType_ptr discr,
			      CORBA::UnionMember &m,
			      CORBA::UnionMemberSeq &members );
  void emitClientUnion( CORBA::UnionDef_ptr u );
  void emitClientAlias( CORBA::AliasDef_ptr a );

  void emitServer();

  void emit_type_code( CORBA::TypeCode_ptr tc );
  void emit_type_code_name( CORBA::IDLType_ptr t );
  void emit_idl_type_name( CORBA::IDLType_ptr t);

  void emit_type_for_param( CORBA::IDLType_ptr t,
			    CORBA::ParameterMode mode );
  void emit_type_for_result( CORBA::IDLType_ptr t );
  void emit_sequence( CORBA::IDLType_ptr t, char *ctx = 0 );
  void emit_array( CORBA::IDLType_ptr t );
  void emit_array_suffix( CORBA::IDLType_ptr t );
  void emit_array_slice_suffix( CORBA::IDLType_ptr t );
  CORBA::ULong compute_num_array_elem( CORBA::TypeCode_ptr tc );
  void emit_type_for_variable( CORBA::IDLType_ptr t, Storage s);
  bool emit_base_type( CORBA::IDLType_ptr t );

  typedef SequenceTmpl<CORBA::TypeCode_ptr> TypeCodeSeq;
  CORBA::TypeCode_ptr resolve_alias( CORBA::TypeCode_ptr tc );
  CORBA::TypeCode_ptr resolve_sequence( CORBA::TypeCode_ptr tc );
  bool is_base_type( CORBA::TypeCode_ptr tc );
  bool is_object( CORBA::TypeCode_ptr tc );
  bool check_for_sequence( TypeCodeSeq& types, CORBA::TypeCode_ptr seq,
			   CORBA::Boolean no_base_types = TRUE);
  bool check_for_array( TypeCodeSeq& types, CORBA::TypeCode_ptr arr );

  char *get_absolute_name (CORBA::Contained_ptr cont, const char *sep);
public:
  CodeGenC( DB &db, bool gen_exceptions, bool query_ir_for_narrow,
	      string &cpp_suffix, string &h_suffix );
  void emit( string &fnbase );
};


#endif
