/*
 *  MICO --- a free CORBA implementation
 *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#include <CORBA.h>
#include <fstream.h>
#include <ctype.h>
#include <stdio.h>
#include "codegen-idl.h"


static char *copyright[] = {
  "/*",
  " *  MICO --- a free CORBA implementation",
  " *  Copyright (C) 1997-98 Kay Roemer & Arno Puder",
  " *",
  " *  This file was automatically generated. DO NOT EDIT!",
  " */"
};

const int copyright_size = sizeof( copyright ) / sizeof( char * );



CodeGenIDL::CodeGenIDL( DB &db, bool emit_repoids )
  : CodeGen( db.get_idl_container() )
{
  _emit_repoids = emit_repoids;
}


void CodeGenIDL::emit( string &fnbase )
{
  string fn = fnbase + ".idl";
  
  ofstream file( fn.c_str() );

  for( int i = 0; i < copyright_size; i++ )
    file << copyright[ i ] << endl;
  
  file << endl;
  o.start_output( file );
  emitIDL();
  o.stop_output();
}

string CodeGenIDL::emit( CORBA::IRObject_ptr obj )
{
  ostrstream idl;

  CORBA::Contained_var contained = CORBA::Contained::_narrow( obj );
  if( !CORBA::is_nil( contained ) ) {
    CORBA::String_var id = contained->id();
    idl << "/*" << endl;
    idl << " * " << id.in() << endl;
    idl << " */" << endl << endl;
  }
  
  o.start_output( idl );
    
  CORBA::Container_var container = CORBA::Container::_narrow( obj );
  if( !CORBA::is_nil( container ) ) {
    switch( container->def_kind() ) {
    case CORBA::dk_Module:
      {
	assert( 0 );
#if 0
	CORBA::ModuleDef_var m = CORBA::ModuleDef::_narrow( container );
	emitIDLModule( m );
#endif
	break;
      }
    case CORBA::dk_Interface:
      {
	CORBA::InterfaceDef_var in =
	  CORBA::InterfaceDef::_narrow( container );
	emitIDLInterface( in );
	break;
      }
    default:
      assert( 0 );
    }
  } else if( !CORBA::is_nil( contained ) ) {
    switch( contained->def_kind() ) {
    case CORBA::dk_Operation:
      {
	CORBA::Contained::Description_var d = contained->describe();
	CORBA::OperationDescription op;
	CORBA::Boolean r = ( d->value >>= op );
	assert (r);
	emitOperation( op );
	break;
      }
    case CORBA::dk_Attribute:
      {
	CORBA::Contained::Description_var d = contained->describe();
	CORBA::AttributeDescription attr;
	CORBA::Boolean r = ( d->value >>= attr );
	assert (r);
	emitAttribute( attr );
	break;
      }
    case CORBA::dk_Struct:
      {
	CORBA::StructDef_var s = CORBA::StructDef::_narrow( contained );
	emitIDLStruct( s );
	break;
      }
    case CORBA::dk_Union:
      {
	CORBA::UnionDef_var u = CORBA::UnionDef::_narrow( contained );
	emitIDLUnion( u );
	break;
      }
    case CORBA::dk_Constant:
      {
	CORBA::ConstantDef_var co = CORBA::ConstantDef::_narrow( contained );
	emitIDLConstant( co );
	break;
      }
    case CORBA::dk_Exception:
      {
	CORBA::ExceptionDef_var e = CORBA::ExceptionDef::_narrow( contained );
	emitIDLException( e );
	break;
      }
    case CORBA::dk_Enum:
      {
	CORBA::EnumDef_var e = CORBA::EnumDef::_narrow( contained );
	emitIDLEnum( e );
	break;
      }
    case CORBA::dk_Alias:
      {
	CORBA::AliasDef_var a = CORBA::AliasDef::_narrow( contained );
	emitIDLAlias( a );
	break;
      }
    default:
      assert( 0 );
    }
  }
  o.stop_output();
  idl << ends;
  string result = idl.str();
  idl.rdbuf()->freeze( 0 );
  return result;
}

void CodeGenIDL::enter_scope( CORBA::Container_ptr c )
{
  vector<CORBA::Contained_var> trace_current_scope;
  vector<CORBA::Contained_var> trace_new_scope;
  CORBA::Container_var         scope;
  bool                         nl;
  
  scope = _current_scope;
  while( scope->def_kind() != CORBA::dk_Repository ) {
    CORBA::Contained_ptr scope_contained = CORBA::Contained::_narrow( scope );
    assert( !CORBA::is_nil( scope_contained ) );
    trace_current_scope.insert( trace_current_scope.begin(), scope_contained );
    scope = scope_contained->defined_in();
  }
  
  scope = CORBA::Container::_duplicate( c );
  while( scope->def_kind() != CORBA::dk_Repository ) {
    CORBA::Contained_ptr scope_contained = CORBA::Contained::_narrow( scope );
    assert( !CORBA::is_nil( scope_contained ) );
    trace_new_scope.insert( trace_new_scope.begin(), scope_contained );
    scope = scope_contained->defined_in();
  }
  
  vector<CORBA::Contained_var>::iterator i, j;
  for( i = trace_current_scope.begin(), j = trace_new_scope.begin();
       i != trace_current_scope.end() && j != trace_new_scope.end() &&
	 (*i)->_is_equivalent( *j );
       i++, j++ ) ;
  
  nl = false;
  while( i != trace_current_scope.end() ) {
    o << BL_CLOSE_SEMI;
    nl = true;
    i++;
  }
  if( nl )
    o << endl << endl;

  nl = false;
  while( j != trace_new_scope.end() ) {
    emit_repoid( *j );
    o << "module ";
    CORBA::Identifier_var name = (*j)->name();
    o << name << " " << BL_OPEN;
    CORBA::Container_var m  = CORBA::Container::_narrow( *j );
    assert( !CORBA::is_nil( m ) );
    emitForwardDcl( m );
    nl = true;
    j++;
  }
  if( nl )
    o << endl;
  
  _current_scope = CORBA::Container::_duplicate( c );
}

void CodeGenIDL::emit_repoid( CORBA::Contained_ptr c )
{
  if( !_emit_repoids )
    return;
  CORBA::String_var name = c->name();
  CORBA::String_var repoid = c->id();
  o << "#pragma ID " << name << " \"" << repoid << "\"" << endl;
}

void CodeGenIDL::add_deferred_decl( CORBA::TypedefDef_ptr t )
{
  int i = _deferred_decl.length();
  _deferred_decl.length( i + 1 );
  _deferred_decl[ i ] = CORBA::TypedefDef::_duplicate( t );
}

void CodeGenIDL::emitAttribute( CORBA::AttributeDescription& attr )
{
  if( attr.mode == CORBA::ATTR_READONLY )
    o << "readonly ";
  o << "attribute ";
  CORBA::IDLType_var type_id = lookup_attribute_by_id( attr.id );
  emit_idl_type( type_id );
  o << " " << attr.name << ";" << endl;
}

void CodeGenIDL::emitOperation( CORBA::OperationDescription& op )
{
  if( op.mode == CORBA::OP_ONEWAY )
    o << "oneway ";
  CORBA::IDLType_var type_id = lookup_result_by_id( op.id );
  emit_idl_type( type_id );
  o << " " << op.name << "(";
  CORBA::ParDescriptionSeq p = op.parameters;
  CORBA::ULong k;
  for( k = 0; k < p.length(); k++ ) {
    if( k != 0 )
      o << ",";
    o << " ";
    switch( p[ k ].mode ) {
    case CORBA::PARAM_IN:
      o << "in ";
      break;
    case CORBA::PARAM_OUT:
      o << "out ";
      break;
    case CORBA::PARAM_INOUT:
      o << "inout ";
      break;
    default:
      assert( 0 );
    }
    emit_idl_type( p[ k ].type_def );
    o << " " << p[ k ].name;
  }
  if( k != 0 )
    o << " ";
  o << ")" << indent;
  
  CORBA::ExcDescriptionSeq exs = op.exceptions;
  if( exs.length() > 0 ) {
    o << endl;
    o << "raises( ";
    for( CORBA::ULong i = 0; i < exs.length(); i++ ) {
      if( i != 0 )
	o << ", ";
      CORBA::Contained_var c = _repo->lookup_id( exs[ i ].id );
      assert( !CORBA::is_nil( c ) );
      CORBA::String_var abs_name = c->absolute_name();
      o << abs_name;
    }
    o << " )";
  }
  CORBA::ContextIdSeq ctx = op.contexts;
  if( ctx.length() > 0 ) {
    o << endl;
    o << "context( ";
    for( CORBA::ULong i = 0; i < ctx.length(); i++ ) {
      if( i != 0 )
	o << ", ";
      o << "\"" << ctx[ i ] << "\"";
    }
    o << " )";
  }
  o << exdent << ";" << endl;
}

void CodeGenIDL::emitPrototypes( CORBA::InterfaceDef_ptr in )
{
  CORBA::InterfaceDef::FullInterfaceDescription_var desc;
  desc = in->describe_interface();

  // Emit attributes
  for( CORBA::ULong j = 0; j < desc->attributes.length(); j++ ) {
    CORBA::AttributeDescription attr;
    attr = desc->attributes[ j ];
    emitAttribute( attr );
  }

  o << endl;
  
  // Emit operations
  for( CORBA::ULong j0 = 0; j0 < desc->operations.length(); j0++ ) {
    CORBA::OperationDescription op;
    op = desc->operations[ j0 ];
    emitOperation( op );
  }
}


void CodeGenIDL::emitLocalDecls( CORBA::InterfaceDef_ptr in )
{
  CORBA::ContainedSeq_var c;
  c = in->contents( CORBA::dk_all, 1 );
  for( CORBA::ULong i = 0; i < c->length(); i++ ) {
    emit_repoid( c[ i ] );
    switch( c[ i ]->def_kind() ) {
    case CORBA::dk_Operation:
    case CORBA::dk_Attribute:
      // Operation & attribute declarations are generated in emitPrototypes()
      break;
    case CORBA::dk_Struct:
      {
	CORBA::StructDef_var s = CORBA::StructDef::_narrow( c[ i ] );
	emitIDLStruct( s );
	break;
      }
    case CORBA::dk_Union:
      {
	CORBA::UnionDef_var u = CORBA::UnionDef::_narrow( c[ i ] );
	emitIDLUnion( u );
	break;
      }
    case CORBA::dk_Constant:
      {
	CORBA::ConstantDef_var co = CORBA::ConstantDef::_narrow( c[ i ] );
	emitIDLConstant( co );
	break;
      }
    case CORBA::dk_Exception:
      {
	CORBA::ExceptionDef_var e = CORBA::ExceptionDef::_narrow( c[ i ] );
	emitIDLException( e );
	break;
      }
    case CORBA::dk_Enum:
      {
	CORBA::EnumDef_var e = CORBA::EnumDef::_narrow( c[ i ] );
	emitIDLEnum( e );
	break;
      }
    case CORBA::dk_Alias:
      {
	CORBA::AliasDef_var a = CORBA::AliasDef::_narrow( c[ i ] );
	emitIDLAlias( a );
	break;
      }
    default:
      assert( 0 );
    }
  }
}

void CodeGenIDL::emitForwardDcl( CORBA::Container_ptr con )
{
  // Check if we already emitted the forward decls for this scope
  CORBA::String_var repo_id;
  if( con->def_kind() == CORBA::dk_Repository ) {
    // Generate pseudo repo id
    repo_id = (const char *) "__pseudo_repo_id";
  } else {
    CORBA::Contained_var c = CORBA::Contained::_narrow( con );
    assert( !CORBA::is_nil( c ) );
    repo_id = c->id();
  }
  if( _forward_dcls.find( string( repo_id ) ) != _forward_dcls.end() )
    return;
  
  _forward_dcls.insert( string( repo_id ) );
  
  CORBA::ContainedSeq_var c;
  c = con->contents( CORBA::dk_Interface, 1 );

  if( c->length() > 0 )
    o << endl;
  
  for( CORBA::ULong i = 0; i < c->length(); i++ ) {
    assert( c[ i ]->def_kind() == CORBA::dk_Interface );
    CORBA::String_var name = c[ i ]->name();
    o << "interface " << name << ";" << endl;
  }
  
  if( c->length() > 0 )
    o << endl;
}

void CodeGenIDL::emitIDL()
{
  _current_scope = CORBA::Container::_duplicate( _container );
  
  emitForwardDcl( _container );

  for( CORBA::ULong i = 0; i < _idl_objs.length(); i++ ) {

    CORBA::Contained_var c = CORBA::Contained::_narrow( _idl_objs[ i ] );
    if( !CORBA::is_nil( c ) ) {
      CORBA::Container_var new_scope = c->defined_in();
      enter_scope( new_scope );
      
      emit_repoid( c );

      switch( c->def_kind() ) {
      case CORBA::dk_Interface:
	{
	  CORBA::InterfaceDef_var in = CORBA::InterfaceDef::_narrow( c );
	  emitIDLInterface( in );
	  break;
	}
      case CORBA::dk_Struct:
	{
	  CORBA::StructDef_var s = CORBA::StructDef::_narrow( c );
	  emitIDLStruct( s );
	  break;
	}
      case CORBA::dk_Union:
	{
	  CORBA::UnionDef_var u = CORBA::UnionDef::_narrow( c );
	  emitIDLUnion( u );
	  break;
	}
      case CORBA::dk_Constant:
	{
	  CORBA::ConstantDef_var co = CORBA::ConstantDef::_narrow( c );
	  emitIDLConstant( co );
	  break;
	}
      case CORBA::dk_Exception:
	{
	  CORBA::ExceptionDef_var e = CORBA::ExceptionDef::_narrow( c );
	  emitIDLException( e );
	  break;
	}
      case CORBA::dk_Enum:
	{
	  CORBA::EnumDef_var e = CORBA::EnumDef::_narrow( c );
	  emitIDLEnum( e );
	  break;
	}
      case CORBA::dk_Alias:
	{
	  CORBA::AliasDef_var a = CORBA::AliasDef::_narrow( c );
	  emitIDLAlias( a );
	  break;
	}
      default:
	cout << c->def_kind() << endl;
	assert( 0 );
      }
    }
  }

  // Close open modules
  enter_scope( _container );
}

void CodeGenIDL::emitIDLInterface( CORBA::InterfaceDef_ptr in )
{
  assert( in->def_kind() == CORBA::dk_Interface );
  CORBA::String_var n = in->name();
  string className = (const char *) n;
  
  o << "interface " << className;
 
  CORBA::InterfaceDefSeq_var bases = in->base_interfaces();
  if( bases->length() != 0 ) {
    // This interface *has* base interfaces
    o << " :" << indent << endl;
    for( CORBA::ULong i = 0; i < bases->length(); i++ ) {
      CORBA::String_var base_name = bases[ i ]->absolute_name();
      o << base_name;
      if( i + 1 != bases->length() )
	o << ",";
      o << endl;
    }
    o << exdent;
  } else
    o << endl;
  
  o << BL_OPEN;
  
  // Emit local declarations
  emitLocalDecls( in );
  
  // Generate prototypes
  emitPrototypes( in );

  o << BL_CLOSE_SEMI << endl;
}

void CodeGenIDL::emitIDLStruct( CORBA::StructDef_ptr s, bool emit_semicolon )
{
  assert( s->def_kind() == CORBA::dk_Struct );

  CORBA::TypeCode_var tc = s->type();
  if( emit_semicolon && !tc->is_freestanding() ) {
    add_deferred_decl( s );
    return;
  }
  
  CORBA::String_var name = s->name();
  
  o << "struct " << name << " " << BL_OPEN;
  CORBA::StructMemberSeq_var m = s->members();
  for( CORBA::ULong i = 0; i < m->length(); i++ ) {
    emit_idl_type( m[ i ].type_def );
    o << " " << m[ i ].name;
    emit_array_suffix( m[ i ].type_def );
    o << ";" << endl;
  }
  o << exdent << "}";
  if( emit_semicolon )
    o << ";" << endl << endl;
}

void CodeGenIDL::emitIDLUnion( CORBA::UnionDef_ptr u, bool emit_semicolon )
{
  assert( u->def_kind() == CORBA::dk_Union );

  CORBA::TypeCode_var tc = u->type();
  if( emit_semicolon && !tc->is_freestanding() ) {
    add_deferred_decl( u );
    return;
  }
  
  CORBA::String_var name = u->name();
  
  o << "union " << name << " switch( ";
  CORBA::IDLType_var discr = u->discriminator_type_def();
  emit_idl_type( discr );
  o << " ) " << BL_OPEN;
  CORBA::UnionMemberSeq_var m = u->members();
  tc = discr->type();
  for( CORBA::ULong i = 0; i < m->length(); i++ ) {
    CORBA::TypeCode_var label_tc = m[ i ].label.type();
    if( is_octet( label_tc ) )
      o << "default:";
    else {
      o << "case ";
      if( is_enum( tc ) ) {
	CORBA::String_var s;
	CORBA::ULong idx;
	CORBA::Boolean r = m[ i ].label.enum_get( idx );
	assert (r);
	s = (const char *) tc->unalias()->member_name( idx );
	o << s;
      } else
	o << &m[ i ].label;
      o << ":";
      if( i + 1 != m->length() &&
	  strcmp( m[ i ].name, m[ i + 1 ].name ) == 0 ) {
	o << endl;
	continue;
      }
    }
    o << " ";
    emit_idl_type( m[ i ].type_def );
    o << " " << m[ i ].name;
    emit_array_suffix( m[ i ].type_def );
    o << ";" << endl;
  }
  o << BL_CLOSE_SEMI << endl;
}

void CodeGenIDL::emitIDLConstant( CORBA::ConstantDef_ptr c )
{
  assert( c->def_kind() == CORBA::dk_Constant );
  CORBA::String_var name = c->name();
  o << "const ";
  CORBA::IDLType_var type = c->type_def();
  CORBA::TypeCode_var tc = type->type();
  if( tc->kind() == CORBA::tk_fixed )
    o << "fixed";
  else
    emit_base_type( type );
  o << " " << name << " = ";
  CORBA::Any* a = c->value();
  o << a;
  delete a;
  if( tc->kind() == CORBA::tk_fixed )
    o << "D";
  o << ";" << endl;
}

void CodeGenIDL::emitIDLException( CORBA::ExceptionDef_ptr e )
{
  assert( e->def_kind() == CORBA::dk_Exception );

  CORBA::String_var name = e->name();
  
  o << "exception " << name << " " << BL_OPEN;
  CORBA::StructMemberSeq_var m = e->members();
  for( CORBA::ULong i = 0; i < m->length(); i++ ) {
    emit_idl_type( m[ i ].type_def );
    o << " " << m[ i ].name;
    emit_array_suffix( m[ i ].type_def );
    o << ";" << endl;
  }
  o << BL_CLOSE_SEMI << endl;
}

void CodeGenIDL::emitIDLEnum( CORBA::EnumDef_ptr e )
{
  assert( e->def_kind() == CORBA::dk_Enum );

  CORBA::String_var name = e->name();
  o << "enum " << name << " " << BL_OPEN;

  CORBA::EnumMemberSeq_var m = e->members();
  
  for( CORBA::ULong i = 0; i < m->length(); i++ ) {
    if( i != 0 )
      o << "," << endl;
    o << m[ i ];
  }
  o << endl;
  o << BL_CLOSE_SEMI << endl;
}

void CodeGenIDL::emitIDLAlias( CORBA::AliasDef_ptr a )
{
  assert( a->def_kind() == CORBA::dk_Alias );
  
  CORBA::IDLType_var orig = a->original_type_def();
  assert( !CORBA::is_nil( orig ) );
  
  CORBA::TypeCode_var orig_tc = orig->type();
  
  CORBA::String_var name = a->name();
  
  if( is_string( orig_tc ) ) {
    o << "typedef string " << name;
    emit_array_suffix( orig );
    o << ";" << endl << endl;
  } else if( is_objref( orig_tc ) ) {
    CORBA::String_var abs_name;
    
    if( orig_tc->kind() == CORBA::tk_alias ) {
      // we have an alias of an alias
      CORBA::AliasDef_var alias = CORBA::AliasDef::_narrow( orig );
      assert( !CORBA::is_nil( alias ) );
      abs_name = alias->absolute_name();
    } else {
      // gotta be an interface
      assert( orig_tc->kind() == CORBA::tk_objref );
      CORBA::InterfaceDef_var in = CORBA::InterfaceDef::_narrow( orig );
      if( !CORBA::is_nil( in ) )
	abs_name = in->absolute_name();
      else
	abs_name = (const char *) "Object";
    }

    o << "typedef " << abs_name << " " << name;
    emit_array_suffix( orig );
    o << ";" << endl << endl;
  } else {
    o << "typedef ";
    emit_idl_type( orig );
    o << " " << name;
    emit_array_suffix( orig );
    o << ";" << endl;
  }
  o << endl;
}

void CodeGenIDL::emit_idl_type_name( CORBA::IDLType_ptr t )
{
  CORBA::String_var tmp;
  string abs_name;
  
  switch( t->def_kind() ) {
  case CORBA::dk_Struct:
  case CORBA::dk_Union:
  case CORBA::dk_Enum:
  case CORBA::dk_Alias:
  case CORBA::dk_Interface:
    {
      CORBA::Contained_var con = CORBA::Contained::_narrow( t );
      tmp = con->absolute_name();
      abs_name = tmp;
      break;
    }
  case CORBA::dk_Primitive:
    {
      CORBA::PrimitiveDef_var p = CORBA::PrimitiveDef::_narrow( t );
      if( p->kind() == CORBA::pk_any )
	o << "any";
      else if( p->kind() == CORBA::pk_objref )
	o << "Object";
      else
	assert( 0 );
      return;
    }
  default:
    cout << t->def_kind() << endl;
    assert( 0 );
  }
  if( abs_name[ 0 ] == ':' )
    abs_name = abs_name.substr( 2 );
  o << (char *) abs_name.c_str();
}

void CodeGenIDL::emit_array_suffix( CORBA::IDLType_ptr t )
{
  CORBA::TypeCode_var tc = t->type();
  assert( !CORBA::is_nil( tc ) );

  while( tc->kind() == CORBA::tk_array ) {
    o << "[ " << tc->length() << " ]";
    tc = tc->content_type();
  }
}

void CodeGenIDL::emit_array( CORBA::IDLType_ptr type )
{
  assert( type->def_kind() == CORBA::dk_Array );
  CORBA::ArrayDef_var a;
  CORBA::IDLType_var idl_type = CORBA::IDLType::_duplicate( type );
  while( idl_type->def_kind() == CORBA::dk_Array ) {
    a = CORBA::ArrayDef::_narrow( idl_type );
    assert( !CORBA::is_nil( a ) );
    idl_type = a->element_type_def();
  }
  emit_idl_type( idl_type );
}

void CodeGenIDL::emit_sequence( CORBA::IDLType_ptr type )
{
  assert( type->def_kind() == CORBA::dk_Sequence );
  CORBA::SequenceDef_var s = CORBA::SequenceDef::_narrow( type );
  CORBA::IDLType_var idl_type = s->element_type_def();
  
  o << "sequence<";
  emit_idl_type( idl_type );
  
  CORBA::ULong b = s->bound();
  if( b > 0 )
    o << ", " << b;
  
  o << " >";
}

bool CodeGenIDL::emit_deferred_decl( CORBA::IDLType_ptr t )
{
  CORBA::TypedefDef_var type = CORBA::TypedefDef::_narrow( t );
  if( CORBA::is_nil( type ) )
    return false;
  for( CORBA::ULong i = 0; i < _deferred_decl.length(); i++ ) {
    if( type->_is_equivalent( _deferred_decl[ i ] ) ) {
      // Erase this deferred decl
      for( CORBA::ULong j = i; j < _deferred_decl.length() - 1; j++ )
	_deferred_decl[ j ] =
	  CORBA::TypedefDef::_duplicate( _deferred_decl[ j + 1 ] );
      _deferred_decl.length( _deferred_decl.length() - 1 );
      
      CORBA::StructDef_var s = CORBA::StructDef::_narrow( type );
      if( !CORBA::is_nil( s ) ) {
	emitIDLStruct( s, false );
	return true;
      }
      CORBA::UnionDef_var u = CORBA::UnionDef::_narrow( type );
      if( !CORBA::is_nil( u ) ) {
	emitIDLUnion( u, false );
	return true;
      }
      assert( 0 );
    }
  }
  return false;
}

void CodeGenIDL::emit_idl_type( CORBA::IDLType_ptr t )
{
  if( emit_deferred_decl( t ) )
    return;

  CORBA::String_var tmp;
  
  CORBA::TypeCode_var tc = t->type();
  assert( !CORBA::is_nil( tc ) );

  if( is_alias( tc ) ) {
    emit_idl_type_name( t );
    return;
  }

  CORBA::PrimitiveDef_var prim = CORBA::PrimitiveDef::_narrow( t );
  if( !CORBA::is_nil( prim ) && prim->kind() == CORBA::pk_objref ) {
    o << "Object";
    return;
  }
  
  if( is_objref( tc ) ) {
    CORBA::InterfaceDef_var in = CORBA::InterfaceDef::_narrow( t );
    tmp = in->absolute_name();
    string abs_name;
    abs_name = tmp;
    abs_name = abs_name.substr( 2 );
    o << abs_name;
    return;
  }

  if( !emit_base_type( t ) ) {
    switch( tc->kind() ) {
    case CORBA::tk_objref:
    case CORBA::tk_struct:
    case CORBA::tk_union:
    case CORBA::tk_enum:
    case CORBA::tk_alias:
    case CORBA::tk_any:
      emit_idl_type_name( t );
      break;
    case CORBA::tk_sequence:
      emit_sequence( t );
      break;
    case CORBA::tk_array:
      emit_array( t );
      break;
    case CORBA::tk_TypeCode:
      o << "TypeCode";
      break;
    default:
      assert( 0 );
    }
  }
}

bool CodeGenIDL::emit_base_type( CORBA::IDLType_ptr t )
{
  CORBA::TypeCode_var tc = t->type();
  assert( !CORBA::is_nil( tc ) );

  // Mapping of Base Data Types [16.5]
  switch( tc->kind() ) {
  case CORBA::tk_void:
    o << "void";
    break;
  case CORBA::tk_boolean:
    o << "boolean";
    break;
  case CORBA::tk_char:
    o << "char";
    break;
  case CORBA::tk_wchar:
    o << "wchar";
    break;
  case CORBA::tk_octet:
    o << "octet";
    break;
  case CORBA::tk_ushort:
    o << "unsigned short";
    break;
  case CORBA::tk_ulong:
    o << "unsigned long";
    break;
  case CORBA::tk_short:
    o << "short";
    break;
  case CORBA::tk_long:
    o << "long";
    break;
  case CORBA::tk_longlong:
    o << "long long";
    break;
  case CORBA::tk_ulonglong:
    o << "unsigned long long";
    break;
  case CORBA::tk_float:
    o << "float";
    break;
  case CORBA::tk_double:
    o << "double";
    break;
  case CORBA::tk_longdouble:
    o << "long double";
    break;
  case CORBA::tk_string:
    o << "string";
    if( tc->length() != 0 )
      o << "<" << tc->length() << ">";
    break;
  case CORBA::tk_wstring:
    o << "wstring";
    if( tc->length() != 0 )
      o << "<" << tc->length() << ">";
    break;
  case CORBA::tk_fixed:
    o << "fixed<" << tc->fixed_digits() << "," <<
      tc->fixed_scale() << ">";
    break;
  case CORBA::tk_alias:
    if( is_base_type( tc ) ) {
      emit_idl_type_name( t );
      return true;
    } else
      return false;
    break;
  default:
    return false;
  }
  return true;
}

