/*
 *  MICO --- a free CORBA implementation
 *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */


#include <CORBA.h>
#include <stdio.h>
#include <iostream.h>
#include <unistd.h>
#include <mico/ir_impl.h>
#include <mico/util.h>
#include "params.h"



IDLParam::IDLParam( int argc, char *argv[] )
{
  _argc = argc;
  _argv = argv;
  set_defaults();
  parse_params();
  check_params();
}

void
IDLParam::set_defaults()
{
  cpp_suffix = "cc";
  c_suffix = "c";
  hh_suffix = "h";
  h_suffix = "h";
  relative_paths = false;
  feed_ir = false;
  feed_included_defs = false;
  codegen_cpp = true;
  cpp_impl = false;
  codegen_c = false;
  codegen_idl = false;
  emit_repoids = false;
  query_ir_for_narrow = false;
#ifdef HAVE_EXCEPTIONS
  no_exceptions = false;
#else
  no_exceptions = true;
#endif
  pseudo = false;
  boa = true;
  poa = false;
  sii = false;
}

void
IDLParam::parse_params()
{
  MICOGetOpt::OptMap opts;
  opts["--feed-ir"]               = "";
  opts["--feed-included-defs"]    = "";
  opts["--no-exceptions"]         = "";
  opts["--codegen-c++"]           = "";
  opts["--no-codegen-c++"]        = "";
  opts["--codegen-idl"]           = "";
  opts["--no-codegen-idl"]        = "";
  opts["--codegen-c"]		  = "";
  opts["--no-codegen-c"]	  = "";
  opts["--c++-suffix"]            = "arg-expected";
  opts["--c++-impl"]              = "";
  opts["--h-suffix"]              = "arg-expected";
  opts["--hh-suffix"]             = "arg-expected";
  opts["--relative-paths"]        = "";
  opts["--emit-repoids"]          = "";
  opts["--query-ir-for-narrow"]   = "";
  opts["--query-server-for-narrow"] = "";
  opts["--repo-id"]               = "arg-expected";
  opts["--name"]                  = "arg-expected";
  opts["--version"]               = "";
  opts["--help"]                  = "";
  opts["-I"]                      = "arg-expected";
  opts["-D"]                      = "arg-expected";
  opts["--poa"]                   = "";
  opts["--no-poa"]                = "";
  opts["--boa"]                   = "";
  opts["--no-boa"]                = "";
  opts["--pseudo"]                = "";
  opts["--sii"]                   = "";
  
  MICOGetOpt opt_parser( opts );
  if( !opt_parser.parse( _argc, _argv ) )
    usage();

  for( MICOGetOpt::OptVec::const_iterator i = opt_parser.opts().begin();
       i != opt_parser.opts().end(); i++ ) {
    string arg = (*i).first;
    string val = (*i).second;

    if (arg == "-I") {
      cpp_options += " ";
      cpp_options += arg + val;
      inc_paths.push_back (val);
    } else if (arg == "-D") {
      cpp_options += " ";
      cpp_options += arg + val;
    } else if (arg == "--feed-ir") {
      feed_ir = true;
    } else if( arg == "--feed-included-defs" ) {
      feed_included_defs = true;
    } else if (arg == "--no-exceptions") {
      no_exceptions = true;
    } else if (arg == "--codegen-c++") {
      codegen_cpp = true;
    } else if (arg == "--no-codegen-c++") {
      codegen_cpp = false;
    } else if (arg == "--codegen-idl") {
      codegen_idl = true;
    } else if (arg == "--no-codegen-idl") {
      codegen_idl = false;
    } else if (arg == "--codegen-c") {
      codegen_c = true;
    } else if (arg == "--no-codegen-c") {
      codegen_c = false;
    } else if( arg == "--c++-suffix" ) {
      cpp_suffix = val;
    } else if( arg == "--c++-impl" ) {
      cpp_impl = true;
    } else if( arg == "--h-suffix" ) {
      h_suffix = val;
    } else if( arg == "--hh-suffix" ) {
      hh_suffix = val;
    } else if( arg == "--relative-paths" ) {
      relative_paths = true;
    } else if (arg == "--emit-repoids") {
      emit_repoids = true;
    } else if( arg == "--query-ir-for-narrow" ) {
      cerr << "warning: --query-ir-for-narrow will go away," << endl;
      cerr << "         use --query-server-for-narrow instead." << endl;
      query_ir_for_narrow = true;
    } else if( arg == "--query-server-for-narrow" ) {
      query_ir_for_narrow = true;
    } else if (arg == "--repo-id") {
      repo_id = val;
    } else if (arg == "--name") {
      name = val;
    } else if (arg == "--boa") {
      boa = true;
    } else if (arg == "--no-boa") {
      boa = false;
    } else if (arg == "--poa") {
      poa = true;
    } else if (arg == "--no-poa") {
      poa = false;
    } else if (arg == "--pseudo") {
      pseudo = true;
    } else if( arg == "--sii" ) {
      sii = true;
    } else if (arg == "--version") {
      cout << "MICO Version " << MICO_VERSION << endl;
      exit( 1 );
    } else if (arg == "--help") {
      usage();
    } else {
      usage();
    }
  }
  if (_argc > 2) {
    usage();
  }
  if (_argc == 2) {
    if (name == "")
      name = _argv[1];
    file = _argv[1];
  }
}

void
IDLParam::check_params()
{
#ifdef HAVE_EXCEPTIONS
  if( no_exceptions && !codegen_cpp ) {
    cerr << "error: --no-exceptions can only be used in conjunction ";
    cerr << "with --codegen-c++" << endl;
    exit( 1 );
  }
#endif
  if( repo_id != "" && feed_ir ) {
    cerr << "error: --repo-id and --feed-ir can not be used together" << endl;
    exit( 1 );
  }
  if( feed_included_defs && !feed_ir ) {
    cerr << "error: use --feed-included-defs only in conjunction "
	 << "with --feed-ir" << endl;
    exit( 1 );
  }
  if( repo_id != "" && file != "" ) {
    cerr << "error: --repo-id and file name can not be used together" << endl;
    exit( 1 );
  }
  if( repo_id == "" && file == "" ) {
    cerr << "error: must either specify a file name or --repo-id";
    cerr << endl;
    exit( 1 );
  }
  if( repo_id != "" && (!codegen_cpp && !codegen_idl) ) {
    cerr << "error: must either specify --codegen-c++ or --codegen-idl";
    cerr << endl;
    exit( 1 );
  }
  if( repo_id != "" && (!codegen_cpp && !codegen_idl && !codegen_c) ) {
    cerr << "error: must either specifiy --codegen-c++, --codegen-idl, "
	 << "or --codegen-c" << endl;
    exit( 1 );
  }
  if( repo_id != "" && name == "" ) {
    cerr << "error: use --name to specify a filename prefix";
    cerr << endl;
    exit( 1 );
  }
  if( emit_repoids && !codegen_idl ) {
    cerr << "error: use --emit-repoids only in conjunction with --codegen-idl";
    cerr << endl;
    exit( 1 );
  }
  if( query_ir_for_narrow && !codegen_cpp ) {
    cerr << "error: use --query-ir-for-narrow only in conjunction ";
    cerr << "with --codegen-c++";
    cerr << endl;
    exit( 1 );
  }
  if( file != "" && !feed_ir ) {
    if( !codegen_cpp && !codegen_idl && !codegen_c) {
      cerr << "error: must either specify --codegen-c++, --codegen-c, or "
	   << "--codegen-idl" << endl;
      exit( 1 );
    }
  }
  if( file != "" && name == file && codegen_idl ) {
    cerr << "error: use --name to specify a filename prefix";
    cerr << endl;
    exit( 1 );
  }
  if( pseudo && query_ir_for_narrow ) {
    cerr << "error: --query-server-for-narrow incompatible with ";
    cerr << "--pseudo";
    cerr << endl;
    exit( 1 );
  }
  if( pseudo && !codegen_cpp ) {
    cerr << "error: --pseudo only works with --codegen-c++";
    cerr << endl;
    exit( 1 );
  }
  if( cpp_impl && !codegen_cpp ) {
    cerr << "error: --c++-impl can only be used in conjunction ";
    cerr << "with --codegen-c++" << endl;
    exit( 1 );
  }
}

void
IDLParam::usage()
{
  cerr << "usage: " << _argv[ 0 ] << " [<options>] [<file>]" << endl;
  cerr << "possible <options> are:" << endl;
  cerr << "    --help" << endl;
  cerr << "    --version" << endl;
  cerr << "    --feed-ir" << endl;
  cerr << "    --no-exceptions" << endl;
  cerr << "    --codegen-c++" << endl;
  cerr << "    --no-codegen-c++" << endl;
  cerr << "    --codegen-idl" << endl;
  cerr << "    --no-codegen-idl" << endl;
  cerr << "    --codegen-c" << endl;
  cerr << "    --no-codegen-c" << endl;
  cerr << "    --c++-suffix <filename-suffix>" << endl;
  cerr << "    --c++-impl" << endl;
  cerr << "    --h-suffix <filename-suffix>" << endl;
  cerr << "    --hh-suffix <filename-suffix>" << endl;
  cerr << "    --relative-paths" << endl;
  cerr << "    --emit-repoids" << endl;
  cerr << "    --query-ir-for-narrow" << endl;
  cerr << "    --repo-id <repository-id>" << endl;
  cerr << "    --name <filename-prefix>" << endl;
  cerr << "    --poa" << endl;
  cerr << "    --no-poa" << endl;
  cerr << "    --boa" << endl;
  cerr << "    --no-boa" << endl;
  cerr << "    --pseudo" << endl;
  exit( 1 );
}
