/*
 *  MICO --- a free CORBA implementation
 *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

%{

#include <CORBA.h>

#include <iostream.h>
#ifdef _WINDOWS
#include <malloc.h>
#endif

#include "parser.h"
#include "parsenode.h"
#include "prepro.h"
#include <mico/template_impl.h>

extern int yylex();

void yyerror( char *s )
{
  theParser->parse_error( idl_lexFile, s, idl_line_no );
}

%}


%union 
{
  char              *ident;
  ParseNode         *node;
  CORBA::LongLong   _int;
  char             *_str;
  CORBA::Char       _char;
  CORBA::LongDouble _float;
  CORBA::LongDouble _fixed;
}


%token T_AMPERSAND
%token T_ANY
%token T_ASTERISK
%token T_ATTRIBUTE
%token T_BOOLEAN
%token T_CASE
%token T_CHAR
%token <_char> T_CHARACTER_LITERAL
%token T_CIRCUMFLEX
%token T_COLON
%token T_COMMA
%token T_CONST
%token T_CONTEXT
%token T_DEFAULT
%token T_DOUBLE
%token T_ENUM
%token T_EQUAL
%token T_EXCEPTION
%token T_FALSE
%token T_FIXED
%token <_fixed> T_FIXED_PT_LITERAL
%token T_FLOAT
%token <_float> T_FLOATING_PT_LITERAL
%token T_GREATER_THAN_SIGN
%token <ident> T_IDENTIFIER
%token T_IN
%token T_INOUT
%token <_int> T_INTEGER_LITERAL
%token T_INTERFACE
%token T_LEFT_CURLY_BRACKET
%token T_LEFT_PARANTHESIS
%token T_LEFT_SQUARE_BRACKET
%token T_LESS_THAN_SIGN
%token T_LONG
%token T_MINUS_SIGN
%token T_MODULE
%token T_OCTET
%token T_ONEWAY
%token T_OUT
%token T_PERCENT_SIGN
%token T_PLUS_SIGN
%token T_PRINCIPAL
%token T_RAISES
%token T_READONLY
%token T_RIGHT_CURLY_BRACKET
%token T_RIGHT_PARANTHESIS
%token T_RIGHT_SQUARE_BRACKET
%token T_SCOPE
%token T_SEMICOLON
%token T_SEQUENCE
%token T_SHIFTLEFT
%token T_SHIFTRIGHT
%token T_SHORT
%token T_SOLIDUS
%token T_STRING
%token <_str> T_STRING_LITERAL
%token <_str> T_PRAGMA
%token T_STRUCT
%token T_SWITCH
%token T_TILDE
%token T_TRUE
%token T_OBJECT
%token T_TYPEDEF
%token T_UNION
%token T_UNSIGNED
%token T_VERTICAL_LINE
%token T_VOID
%token T_WCHAR
%token T_WSTRING
%token T_UNKNOWN


%type <node>   definitions
%type <node>   definition
%type <node>   interface
%type <node>   interface_dcl
%type <node>   interface_header
%type <node>   interface_body
%type <node>   exports
%type <node>   export
%type <node>   op_dcl
%type <node>   op_attribute
%type <node>   op_type_spec
%type <node>   parameter_dcls
%type <node>   except_dcl
%type <node>   raises_expr
%type <node>   context_expr
%type <node>   param_attribute
%type <node>   param_type_spec
%type <node>   floating_pt_type
%type <node>   fixed_pt_type
%type <node>   fixed_pt_const_type
%type <node>   integer_type
%type <node>   signed_int
%type <node>   signed_long_int
%type <node>   signed_longlong_int
%type <node>   signed_short_int
%type <node>   unsigned_int
%type <node>   unsigned_long_int
%type <node>   unsigned_longlong_int
%type <node>   unsigned_short_int
%type <node>   char_type
%type <node>   wide_char_type
%type <node>   boolean_type
%type <node>   octet_type
%type <node>   base_type_spec
%type <node>   param_dcl
%type <node>   simple_declarator
%type <node>   complex_declarator
%type <node>   array_declarator
%type <node>   fixed_array_sizes
%type <node>   fixed_array_size
%type <node>   param_dcls
%type <node>   const_exp
%type <node>   const_type
%type <node>   const_dcl
%type <node>   or_expr
%type <node>   xor_expr
%type <node>   and_expr
%type <node>   shift_expr
%type <node>   add_expr
%type <node>   mult_expr
%type <node>   unary_expr
%type <node>   primary_expr
%type <node>   literal
%type <node>   positive_int_const
%type <node>   string_type
%type <node>   wide_string_type
%type <node>   type_dcl
%type <node>   struct_type
%type <node>   member_list
%type <node>   member
%type <node>   members
%type <node>   type_spec
%type <node>   simple_type_spec
%type <node>   constr_type_spec
%type <node>   declarators
%type <node>   declarator
%type <node>   scoped_name
%type <node>   template_type_spec
%type <node>   enumerator
%type <node>   enumerators
%type <node>   enum_type
%type <node>   module
%type <node>   sequence_type
%type <node>   simple_declarators
%type <node>   attr_dcl
%type <node>   any_type
%type <node>   principal_type
%type <node>   object_type
%type <node>   scoped_names
%type <node>   inheritance_spec
%type <node>   forward_dcl
%type <node>   union_type
%type <node>   switch_type_spec
%type <node>   switch_body
%type <node>   case
%type <node>   case_label
%type <node>   element_spec
%type <node>   string_literals


%%

/*1*/
specification
	: /*empty*/
	  {
	    theParser->setRootNode( NULL );
	  }
	| definitions
          {
	    theParser->setRootNode( $1 );
	  }
	;

	
definitions
	: definition
          {
	    $$ = new ParseNode( definitions, $1 );
	  }
	| definition definitions
          {
	    $$ = new ParseNode( definitions, $1 );
	    $$->setBranch( 2, $2 );
	  }
	;

/*2*/
definition
	: type_dcl T_SEMICOLON
          {
	    $$ = $1;
	  }
	| const_dcl T_SEMICOLON
          {
	    $$ = $1;
	  }
	| except_dcl T_SEMICOLON
          {
	    $$ = $1;
	  }
	| interface T_SEMICOLON
          {
	    $$ = $1;
	  }
	| module T_SEMICOLON
          {
	    $$ = $1;
	  }
        | T_PRAGMA      /*New*/
          {
	    $$ = new ParseNode( t_pragma );
	    $$->setIdent( $1 );
	  }
	;

/*3*/
module
	: T_MODULE T_IDENTIFIER T_LEFT_CURLY_BRACKET
                   definitions T_RIGHT_CURLY_BRACKET
          {
	    $$ = new ParseNode( t_module, $4 );
	    $$->setIdent( $2 );
	  }
	;

/*4*/
interface
	: interface_dcl
          {
	    $$ = $1;
	  }
	| forward_dcl
          {
	    $$ = $1;
	  }
	;

/*5*/
interface_dcl
	: interface_header T_LEFT_CURLY_BRACKET interface_body
                                         T_RIGHT_CURLY_BRACKET
          {
	    $$ = new ParseNode( interface_dcl, $1, $3 );
	  }
	;

/*6*/
forward_dcl
	: T_INTERFACE T_IDENTIFIER
          {
	    $$ = new ParseNode( forward_dcl );
	    $$->setIdent( $2 );
	  }
	;

/*7*/
interface_header
	: T_INTERFACE T_IDENTIFIER
          {
	    $$ = new ParseNode( interface_header );
	    $$->setIdent( $2 );
	  }
	| T_INTERFACE T_IDENTIFIER inheritance_spec 
          {
	    $$ = new ParseNode( interface_header );
	    $$->setIdent( $2 );
	    $$->setBranch( 1, $3 );
	  }
	; 

/*8*/
interface_body
	: /*empty*/
          {
	    $$ = NULL;
	  }
	| exports
          {
	    $$ = $1;
	  }
	;

exports
	: export
          {
	    $$ = new ParseNode( exports, $1 );
	  }
	| export exports
          {
	    $$ = new ParseNode( exports, $1 );
	    $$->setBranch( 2, $2 );
	  }
	;

/*9*/
export
	: type_dcl T_SEMICOLON
          {
	    $$ = $1;
	  }
	| const_dcl T_SEMICOLON 
          {
	    $$ = $1;
	  }
	| except_dcl T_SEMICOLON
          {
	    $$ = $1;
	  }
	| attr_dcl T_SEMICOLON
          {
	    $$ = $1;
	  }
	| op_dcl T_SEMICOLON 
          {
	    $$ = $1;
	  }
        | T_PRAGMA      /*New*/
          {
	    $$ = new ParseNode( t_pragma );
	    $$->setIdent( $1 );
	  }
	;

/*10*/
inheritance_spec
	: T_COLON scoped_names
          {
	    $$ = $2;
	  }
	;

scoped_names
	: scoped_name
          {
	    $$ = new ParseNode( scoped_names, $1 );
	  }
	| scoped_name T_COMMA scoped_names
          {
	    $$ = new ParseNode( scoped_names, $1 );
	    $$->setBranch( 2, $3 );
	  }
	;

/*11*/
scoped_name
	: T_IDENTIFIER
          {
	    $$ = new ParseNode( scoped_name );
	    $$->setIdent( $1 );
	  }
        | T_SCOPE T_IDENTIFIER
          {
	    $$ = new ParseNode( scoped_name );
	    string name = "::";
	    name += $2;
	    CORBA::string_free( $2 );
	    $$->setIdent( CORBA::string_dup( name.c_str() ) );
	  }
	| scoped_name T_SCOPE T_IDENTIFIER
          {
	    $$ = $1;
	    string name = $$->getIdent();
	    name += "::";
	    name += $3;
	    CORBA::string_free( $3 );
	    $$->setIdent( CORBA::string_dup( name.c_str() ) );
	  }
	;

/*12*/
const_dcl
	: T_CONST const_type T_IDENTIFIER T_EQUAL const_exp
          {
	    $$ = new ParseNode( const_dcl, $2, $5 );
	    $$->setIdent( $3 );
	  }
	;

/*13*/
const_type
	: integer_type
          {
	    $$ = $1;
	  }
	| char_type
          {
	    $$ = $1;
	  }
	| wide_char_type
          {
	    $$ = $1;
	  }
	| boolean_type
          {
	    $$ = $1;
	  }
	| floating_pt_type
          {
	    $$ = $1;
	  }
	| string_type
          {
	    $$ = $1;
	  }
	| wide_string_type
          {
	    $$ = $1;
	  }
	| fixed_pt_const_type
          {
	    $$ = $1;
	  }
	| scoped_name 
          {
	    $$ = $1;
	  }
	;

/*14*/
const_exp
	: or_expr
          {
	    $$ = $1;
	  }
	;

/*15*/
or_expr
	: xor_expr
          {
	    $$ = $1;
	  }
	| or_expr T_VERTICAL_LINE xor_expr
          {
	    $$ = new ParseNode( t_vertical_line, $1, $3 );
	  }
	;

/*16*/
xor_expr
	: and_expr
          {
	    $$ = $1;
	  }
	| xor_expr T_CIRCUMFLEX and_expr
          {
	    $$ = new ParseNode( t_circumflex, $1, $3 );
	  }
	;

/*17*/
and_expr
	: shift_expr
          {
	    $$ = $1;
	  }
	| and_expr T_AMPERSAND shift_expr
          {
	    $$ = new ParseNode( t_ampersand, $1, $3 );
	  }
	;

/*18*/
shift_expr
	: add_expr
          {
	    $$ = $1;
	  }
	| shift_expr T_SHIFTRIGHT add_expr
          {
	    $$ = new ParseNode( t_shiftright, $1, $3 );
	  }
	| shift_expr T_SHIFTLEFT add_expr
          {
	    $$ = new ParseNode( t_shiftleft, $1, $3 );
	  }
	;

/*19*/
add_expr
	: mult_expr
          {
	    $$ = $1;
	  }
	| add_expr T_PLUS_SIGN mult_expr
          {
	    $$ = new ParseNode( t_plus_sign, $1, $3 );
	  }
	| add_expr T_MINUS_SIGN mult_expr
          {
	    $$ = new ParseNode( t_minus_sign, $1, $3 );
	  }
	;

/*20*/
mult_expr
	: unary_expr
          {
	    $$ = $1;
	  }
	| mult_expr T_ASTERISK unary_expr
          {
	    $$ = new ParseNode( t_asterik, $1, $3 );
	  }
	| mult_expr T_SOLIDUS unary_expr
          {
	    $$ = new ParseNode( t_solidus, $1, $3 );
	  }
	| mult_expr T_PERCENT_SIGN unary_expr
          {
	    $$ = new ParseNode( t_percent_sign, $1, $3 );
	  }
	;

/*21*/
/*22*/
unary_expr
	: T_MINUS_SIGN primary_expr
          {
	    $$ = new ParseNode( t_minus_sign, $2 );
	  }
	| T_PLUS_SIGN primary_expr
          {
	    $$ = new ParseNode( t_plus_sign, $2 );
	  }
	| T_TILDE primary_expr
          {
	    $$ = new ParseNode( t_tilde, $2 );
	  }
	| primary_expr
          {
	    $$ = $1;
	  }
	;

/*23*/
primary_expr
	: scoped_name
          {
	    $$ = $1;
	  }
	| literal
          {
	    $$ = $1;
	  }
	| T_LEFT_PARANTHESIS const_exp T_RIGHT_PARANTHESIS
          {
	    $$ = $2;
	  }
	;

/*24*/
/*25*/
literal
	: T_INTEGER_LITERAL
          {
	    $$ = new ParseNode( t_integer_literal );
	    $$->setIntegerLiteral( $1 );
	  }
	| T_STRING_LITERAL
          {
	    $$ = new ParseNode( t_string_literal );
	    $$->setStringLiteral( $1 );
	  }
	| T_CHARACTER_LITERAL
          {
	    $$ = new ParseNode( t_character_literal );
	    $$->setCharacterLiteral( $1 );
	  }
	| T_FIXED_PT_LITERAL
          {
	    $$ = new ParseNode( t_fixed_pt_literal );
	    $$->setFixedPtLiteral( $1 );
	  }
	| T_FLOATING_PT_LITERAL
          {
	    $$ = new ParseNode( t_floating_pt_literal );
	    $$->setFloatingPtLiteral( $1 );
	  }
	| T_TRUE  /*boolean_literal*/
          {
	    $$ = new ParseNode( t_boolean_literal );
	    $$->setBooleanLiteral( true );
	  }
	| T_FALSE /*boolean_literal*/
          {
	    $$ = new ParseNode( t_boolean_literal );
	    $$->setBooleanLiteral( false );
	  }
	;

/*26*/
positive_int_const
	: const_exp
          {
	    $$ = $1;
	  }
	;

/*27*/
/*28*/
type_dcl
	: T_TYPEDEF type_spec declarators
          {
	    $$ = new ParseNode( t_typedef, $2, $3 );
	  }
	| struct_type
          {
	    $$ = $1;
	  }
	| union_type
          {
	    $$ = $1;
	  }
	| enum_type
          {
	    $$ = $1;
	  }
	;

/*29*/
type_spec
	: simple_type_spec
          {
	    $$ = $1;
	  }
	| constr_type_spec 
          {
	    $$ = $1;
	  }
	;

/*30*/
simple_type_spec
	: base_type_spec
          {
	    $$ = $1;
	  }
	| template_type_spec
          {
	    $$ = $1;
	  }
	| scoped_name
          {
	    $$ = $1;
	  }
	;

/*31*/
base_type_spec
	: floating_pt_type
          {
	    $$ = $1;
	  }
	| integer_type
          {
	    $$ = $1;
	  }
	| char_type
          {
	    $$ = $1;
	  }
	| wide_char_type
          {
	    $$ = $1;
	  }
	| boolean_type
          {
	    $$ = $1;
	  }
	| octet_type
          {
	    $$ = $1;
	  }
	| any_type
          {
	    $$ = $1;
	  }
	| object_type     /*New*/
          {
	    $$ = $1;
	  }
	| principal_type  /*New*/
          {
	    $$ = $1;
	  }
	;

/*32*/
template_type_spec
	: sequence_type
          {
	    $$ = $1;
	  }
	| string_type
          {
	    $$ = $1;
	  }
	| wide_string_type
          {
	    $$ = $1;
	  }
	| fixed_pt_type
          {
	    $$ = $1;
	  }
	;

/*33*/
constr_type_spec
	: struct_type
          {
	    $$ = $1;
	  }
	| union_type
          {
	    $$ = $1;
	  }
	| enum_type
          {
	    $$ = $1;
	  }
	;

/*34*/
declarators
	: declarator
          {
	    $$ = new ParseNode( declarators, $1 );
	  }
	| declarator T_COMMA declarators
          {
	    $$ = new ParseNode( declarators, $1 );
	    $$->setBranch( 2, $3 );
	  }
	;

/*35*/
declarator
	: simple_declarator
          {
	    $$ = $1;
	  }
	| complex_declarator
          {
	    $$ = $1;
	  }
	;

/*36*/
simple_declarator
	: T_IDENTIFIER
          {
	    $$ = new ParseNode( simple_declarator );
	    $$->setIdent( $1 );
	  }
	;

/*37*/
complex_declarator
	: array_declarator
          {
	    $$ = $1;
	  }
	;

/*38*/
floating_pt_type
	: T_FLOAT
          {
	    $$ = new ParseNode( t_float );
	  }
	| T_DOUBLE
          {
	    $$ = new ParseNode( t_double );
	  }
	| T_LONG T_DOUBLE
          {
	    $$ = new ParseNode( t_long_double );
	  }
	;

/*39*/
integer_type
	: signed_int
          {
	    $$ = $1;
	  }
	| unsigned_int
          {
	    $$ = $1;
	  }
	;

/*40*/
signed_int
	: signed_long_int
          {
	    $$ = $1;
	  }
	| signed_short_int
          {
	    $$ = $1;
	  }
	| signed_longlong_int
          {
	    $$ = $1;
	  }
	;

/*41*/
signed_long_int
	: T_LONG
          {
	    $$ = new ParseNode( t_long );
	  }
	;

/*42*/
signed_short_int
	: T_SHORT
          {
	    $$ = new ParseNode( t_short );
	  }
	;

/*43*/
signed_longlong_int
	: T_LONG T_LONG
          {
	    $$ = new ParseNode( t_longlong );
	  }
	;

/*44*/
unsigned_int
	: unsigned_long_int
          {
	    $$ = $1;
	  }
	| unsigned_short_int
          {
	    $$ = $1;
	  }
	| unsigned_longlong_int
          {
	    $$ = $1;
	  }
	;

/*45*/
unsigned_short_int
	: T_UNSIGNED T_SHORT
          {
	    $$ = new ParseNode( t_unsigned_short );
	  }
	;

/*46*/
unsigned_long_int
	: T_UNSIGNED T_LONG
          {
	    $$ = new ParseNode( t_unsigned_long );
	  }
	;

/*47*/
unsigned_longlong_int
	: T_UNSIGNED T_LONG T_LONG
          {
	    $$ = new ParseNode( t_unsigned_longlong );
	  }
	;

/*48*/
char_type
	: T_CHAR
          {
	    $$ = new ParseNode( t_char );
	  }
	;

/*49*/
wide_char_type
	: T_WCHAR
          {
	    $$ = new ParseNode( t_wide_char );
	  }
	;

/*50*/
boolean_type
	: T_BOOLEAN
          {
	    $$ = new ParseNode( t_boolean );
	  }
	;

/*51*/
octet_type
	: T_OCTET
          {
	    $$ = new ParseNode( t_octet );
	  }
	;

/*52*/
any_type
	: T_ANY
          {
	    $$ = new ParseNode( t_any );
	  }
	;

/*54*/
struct_type
	: T_STRUCT T_IDENTIFIER T_LEFT_CURLY_BRACKET member_list 
                                               T_RIGHT_CURLY_BRACKET
          {
	    $$ = new ParseNode( struct_type, $4 );
	    $$->setIdent( $2 );
	  }
	;

/*55*/
member_list
	: member
          {
	    $$ = new ParseNode( member_list, $1 );
	  }
	| member member_list
          {
	    $$ = new ParseNode( member_list, $1 );
	    $$->setBranch( 2, $2 );
	  }
	;

/*56*/
member
	: type_spec declarators T_SEMICOLON
          {
	    $$ = new ParseNode( member, $1, $2 );
	  }
	;

/*57*/
union_type
	: T_UNION T_IDENTIFIER T_SWITCH T_LEFT_PARANTHESIS
          switch_type_spec T_RIGHT_PARANTHESIS T_LEFT_CURLY_BRACKET
          switch_body T_RIGHT_CURLY_BRACKET
          {
	    $$ = new ParseNode( union_type, $5, $8 );
	    $$->setIdent( $2 );
	  }
	; 

/*58*/
switch_type_spec
	: integer_type
          {
	    $$ = $1;
	  }
	| char_type
          {
	    $$ = $1;
	  }
	| boolean_type
          {
	    $$ = $1;
	  }
	| enum_type
          {
	    $$ = $1;
	  }
	| scoped_name
          {
	    $$ = $1;
	  }
	;

/*59*/
switch_body
	: case
          {
	    $$ = new ParseNode( switch_body, $1 );
	  }
	| case switch_body
          {
	    $$ = new ParseNode( switch_body, $1, $2 );
	  }
	;

/*60*/
case	
	: case_label case
          {
	    $$ = new ParseNode( _case, $1, $2 );
	  }
	| case_label element_spec T_SEMICOLON
          {
	    $$ = new ParseNode( _case, $1 );
	    $$->setBranch( 3, $2 );
	  }
	;

/*61*/
case_label
	: T_CASE const_exp T_COLON 
          {
	    $$ = $2;
	  }
	| T_DEFAULT T_COLON
          {
	    $$ = new ParseNode( t_default );
	  }
	;

/*62*/
element_spec
	: type_spec declarator
          {
	    $$ = new ParseNode( element_spec, $1, $2 );
	  }
	;

/*63*/
enum_type
	: T_ENUM T_IDENTIFIER T_LEFT_CURLY_BRACKET enumerators
                                         T_RIGHT_CURLY_BRACKET
          {
	    $$ = new ParseNode( enum_type, $4 );
	    $$->setIdent( $2 );
	  }
	;

enumerators
	: enumerator
          {
	    $$ = $1;
	  }
	| enumerator T_COMMA enumerators
          {
	    $$ = new ParseNode( enumerators, $1, $3 );
	  }
	;

/*64*/
enumerator
	: T_IDENTIFIER
          {
	    $$ = new ParseNode( enumerator );
	    $$->setIdent( $1 );
	  }
	;

/*65*/
sequence_type
	: T_SEQUENCE T_LESS_THAN_SIGN simple_type_spec T_COMMA
                        positive_int_const T_GREATER_THAN_SIGN
          {
	    $$ = new ParseNode( t_sequence, $3, $5 );
	  }
	| T_SEQUENCE T_LESS_THAN_SIGN simple_type_spec T_GREATER_THAN_SIGN
          {
	    $$ = new ParseNode( t_sequence, $3 );
	  }
	;

/*66*/
string_type
	: T_STRING T_LESS_THAN_SIGN positive_int_const T_GREATER_THAN_SIGN
          {
	    $$ = new ParseNode( t_string, $3 );
	  }
	| T_STRING
          {
	    $$ = new ParseNode( t_string );
	  }
	;

/*67*/
wide_string_type
	: T_WSTRING T_LESS_THAN_SIGN positive_int_const T_GREATER_THAN_SIGN
          {
	    $$ = new ParseNode( t_wide_string, $3 );
	  }
	| T_WSTRING
          {
	    $$ = new ParseNode( t_wide_string );
	  }
	;

/*68*/
array_declarator
	: T_IDENTIFIER fixed_array_sizes
          {
	    $$ = new ParseNode( array_declarator, $2 );
	    $$->setIdent( $1 );
	  }
	;

fixed_array_sizes
	: fixed_array_size
          {
	    $$ = new ParseNode( fixed_array_sizes, $1 );
	  }
	| fixed_array_size fixed_array_sizes
          {
	    $$ = new ParseNode( fixed_array_sizes, $1 );
	    $$->setBranch( 2, $2 );
	  }
	;

/*69*/
fixed_array_size
	: T_LEFT_SQUARE_BRACKET positive_int_const T_RIGHT_SQUARE_BRACKET
          {
	    $$ = $2;
	  }
	;

/*70*/
attr_dcl
	: T_ATTRIBUTE param_type_spec simple_declarators
          {
	    $$ = new ParseNode( t_attribute, $2, $3 );
	  }
	| T_READONLY T_ATTRIBUTE param_type_spec simple_declarators
          {
	    $$ = new ParseNode( t_readonly_attribute, $3, $4 );
	  }
	; 

simple_declarators
	: simple_declarator
          {
	    $$ = new ParseNode( simple_declarators, $1 );
	  }
	| simple_declarator T_COMMA simple_declarators
          {
	    $$ = new ParseNode( simple_declarators, $1 );
	    $$->setBranch( 2, $3 );
	  }
	;

/*71*/
except_dcl
	: T_EXCEPTION T_IDENTIFIER T_LEFT_CURLY_BRACKET members
                                          T_RIGHT_CURLY_BRACKET
          {
	    $$ = new ParseNode( except_dcl, $4 );
	    $$->setIdent( $2 );
	  }
	;

members
	: /*empty*/
          {
	    $$ = NULL;
	  }
	| member members
          {
	    $$ = new ParseNode( member_list, $1 );
	    $$->setBranch( 2, $2 );
	  }
	;

/*72*/
op_dcl
	: op_attribute op_type_spec T_IDENTIFIER parameter_dcls
                                       raises_expr context_expr
          {
	    $$ = new ParseNode( op_dcl, $1, $2, $4, $5, $6 );
	    $$->setIdent( $3 );
	  }
	;

/*73*/
op_attribute
	: /*empty*/
          {
	    $$ = NULL;
	  }
	| T_ONEWAY
          {
	    $$ = new ParseNode( t_oneway );
	  }
	;

/*74*/
op_type_spec	
	: param_type_spec
          {
              $$ = new ParseNode( op_type_spec, $1 );
	  }
	| T_VOID
          {
              $$ = new ParseNode( t_void );
	  }
	;

/*75*/
parameter_dcls
	: T_LEFT_PARANTHESIS param_dcls T_RIGHT_PARANTHESIS
          {
	    $$ = $2;
	  }
	| T_LEFT_PARANTHESIS T_RIGHT_PARANTHESIS
          {
	    $$ = NULL;
	  }
	;

param_dcls
	: param_dcl
          {
	    $$ = new ParseNode( param_dcls, $1 );
	  }
	| param_dcl T_COMMA param_dcls
          {
	    $$ = new ParseNode( param_dcls, $1 );
	    $$->setBranch( 2, $3 );
	  }
	;

/*76*/
param_dcl
	: param_attribute param_type_spec simple_declarator
          {
	    $$ = new ParseNode( param_dcl, $1, $2, $3 );
	  }
	;

/*77*/
param_attribute
	: T_IN
          {
	    $$ = new ParseNode( t_in );
	  }
	| T_OUT
          {
	    $$ = new ParseNode( t_out );
	  }
	| T_INOUT
          {
	    $$ = new ParseNode( t_inout );
	  }
	;

/*78*/
raises_expr
	: /*empty*/
          {
	    $$ = NULL;
	  }
	| T_RAISES T_LEFT_PARANTHESIS scoped_names T_RIGHT_PARANTHESIS
          {
	    $$ = new ParseNode( raises_expr, $3 );
	  }
	;

/*79*/
context_expr
	: /*empty*/
          {
	    $$ = NULL;
	  }
	| T_CONTEXT T_LEFT_PARANTHESIS string_literals T_RIGHT_PARANTHESIS
          {
	    $$ = $3;
	  }
	;

string_literals
	: T_STRING_LITERAL
          {
	    $$ = new ParseNode( string_literals );
	    $$->setIdent( $1 );
	  }
	| T_STRING_LITERAL T_COMMA string_literals
          {
	    $$ = new ParseNode( string_literals, $3 );
	    $$->setIdent( $1 );
	  }
	;

/*80*/
param_type_spec
	: base_type_spec
          {
	    $$ = $1;
	  }
	| string_type
          {
	    $$ = $1;
	  }
	| wide_string_type
          {
	    $$ = $1;
	  }
	| fixed_pt_type
          {
	    $$ = $1;
	  }
	| scoped_name
          {
	    $$ = $1;
	  }
	;

/*81*/
fixed_pt_type
	: T_FIXED T_LESS_THAN_SIGN positive_int_const T_COMMA
              T_INTEGER_LITERAL T_GREATER_THAN_SIGN
          {
	    $$ = new ParseNode( fixed_pt_type, $3 );
	    $$->setIntegerLiteral( $5 );
	  }
	;

/*82*/
fixed_pt_const_type
	: T_FIXED
          {
	    $$ = new ParseNode( fixed_pt_type );
	  }
	;


/* New production for Object */
object_type
	: T_OBJECT
          {
	    $$ = new ParseNode( t_object );
	  }
	;

/* New production for Principal */
principal_type
	: T_PRINCIPAL
          {
	    $$ = new ParseNode( t_principal );
	  }
	;

%%
