/*
 *  MICO --- a free CORBA implementation
 *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */


#include <CORBA.h>
#include <mico/template_impl.h>



CORBA::Boolean
operator<<=( CORBA::Any &a, const SequenceTmpl<CORBA::CharWrapper> &s )
{
  static CORBA::TypeCode_ptr tc = NULL;

  if( tc == NULL )
    tc=CORBA::TypeCode::create_sequence_tc(0,CORBA::_tc_char)->mk_constant();
  a.type( tc );
  if( !a.seq_put_begin( s.length() ) )
    return FALSE;
  for( CORBA::ULong i = 0; i < s.length(); i++ )
    if( !(a <<= CORBA::Any::from_char( ((SequenceTmpl<CORBA::CharWrapper>&)s)[ i ] )) )
      return FALSE;
  return a.seq_put_end();
}

CORBA::Boolean
operator>>=( const CORBA::Any &a, SequenceTmpl<CORBA::CharWrapper> &s )
{
  CORBA::ULong len;

  if( !a.seq_get_begin( len ) )
    return FALSE;
  s.length( len );
  for( CORBA::ULong i = 0; i < len; i++ )
    if( !(a >>= CORBA::Any::to_char( s[ i ] )) )
	return FALSE;
  return a.seq_get_end();
}

CORBA::Boolean
operator<<=( CORBA::Any &a, const SequenceTmpl<CORBA::BooleanWrapper> &s )
{
  static CORBA::TypeCode_ptr tc = NULL;

  if( tc == NULL )
    tc=CORBA::TypeCode::create_sequence_tc(
	0, CORBA::_tc_boolean)->mk_constant();
  a.type( tc );
  if( !a.seq_put_begin( s.length() ) )
    return FALSE;
  for( CORBA::ULong i = 0; i < s.length(); i++ )
    if( !(a <<= CORBA::Any::from_boolean( ((SequenceTmpl<CORBA::BooleanWrapper>&)s)[ i ] )) )
      return FALSE;
  return a.seq_put_end();
}

CORBA::Boolean
operator>>=( const CORBA::Any &a, SequenceTmpl<CORBA::BooleanWrapper> &s )
{
  CORBA::ULong len;

  if( !a.seq_get_begin( len ) )
    return FALSE;
  s.length( len );
  for( CORBA::ULong i = 0; i < len; i++ )
    if( !(a >>= CORBA::Any::to_boolean( s[ i ] )) )
	return FALSE;
  return a.seq_get_end();
}

CORBA::Boolean
operator<<=( CORBA::Any &a, const SequenceTmpl<CORBA::OctetWrapper> &s )
{
  static CORBA::TypeCode_ptr tc = NULL;

  if( tc == NULL )
    tc=CORBA::TypeCode::create_sequence_tc(0,CORBA::_tc_octet)->mk_constant();
  a.type( tc );
  if( !a.seq_put_begin( s.length() ) )
    return FALSE;
  for( CORBA::ULong i = 0; i < s.length(); i++ )
    if( !(a <<= CORBA::Any::from_octet( ((SequenceTmpl<CORBA::OctetWrapper>&)s)[ i ] )) )
      return FALSE;
  return a.seq_put_end();
}

CORBA::Boolean
operator>>=( const CORBA::Any &a, SequenceTmpl<CORBA::OctetWrapper> &s )
{
  CORBA::ULong len;

  if( !a.seq_get_begin( len ) )
    return FALSE;
  s.length( len );
  for( CORBA::ULong i = 0; i < len; i++ )
    if( !(a >>= CORBA::Any::to_octet( s[ i ] )) )
	return FALSE;
  return a.seq_get_end();
}

CORBA::Boolean
operator<<=( CORBA::Any &a, const SequenceTmpl<CORBA::UShort> &s )
{
  static CORBA::TypeCode_ptr tc = NULL;

  if( tc == NULL )
    tc=CORBA::TypeCode::create_sequence_tc(0,CORBA::_tc_ushort)->mk_constant();
  a.type( tc );
  if( !a.seq_put_begin( s.length() ) )
    return FALSE;
  for( CORBA::ULong i = 0; i < s.length(); i++ )
    if( !(a <<= ((SequenceTmpl<CORBA::UShort>&)s)[ i ]) )
      return FALSE;
  return a.seq_put_end();
}

CORBA::Boolean
operator>>=( const CORBA::Any &a, SequenceTmpl<CORBA::UShort> &s )
{
  CORBA::ULong len;

  if( !a.seq_get_begin( len ) )
    return FALSE;
  s.length( len );
  for( CORBA::ULong i = 0; i < len; i++ )
    if( !(a >>= s[ i ]) )
      return FALSE;
  return a.seq_get_end();
}


CORBA::Boolean
operator<<=( CORBA::Any &a, const SequenceTmpl<CORBA::ULong> &s )
{
  static CORBA::TypeCode_ptr tc = NULL;

  if( tc == NULL )
    tc=CORBA::TypeCode::create_sequence_tc(0,CORBA::_tc_ulong)->mk_constant();
  a.type( tc );
  if( !a.seq_put_begin( s.length() ) )
    return FALSE;
  for( CORBA::ULong i = 0; i < s.length(); i++ )
    if( !(a <<= ((SequenceTmpl<CORBA::ULong>&)s)[ i ]) )
      return FALSE;
  return a.seq_put_end();
}

CORBA::Boolean
operator>>=( const CORBA::Any &a, SequenceTmpl<CORBA::ULong> &s )
{
  CORBA::ULong len;

  if( !a.seq_get_begin( len ) )
    return FALSE;
  s.length( len );
  for( CORBA::ULong i = 0; i < len; i++ )
    if( !(a >>= s[ i ]) )
      return FALSE;
  return a.seq_get_end();
}


CORBA::Boolean
operator<<=( CORBA::Any &a, const SequenceTmpl<CORBA::Short> &s )
{
  static CORBA::TypeCode_ptr tc = NULL;

  if( tc == NULL )
    tc=CORBA::TypeCode::create_sequence_tc(0,CORBA::_tc_short)->mk_constant();
  a.type( tc );
  if( !a.seq_put_begin( s.length() ) )
    return FALSE;
  for( CORBA::ULong i = 0; i < s.length(); i++ )
    if( !(a <<= ((SequenceTmpl<CORBA::Short>&)s)[ i ]) )
      return FALSE;
  return a.seq_put_end();
}

CORBA::Boolean
operator>>=( const CORBA::Any &a, SequenceTmpl<CORBA::Short> &s )
{
  CORBA::ULong len;

  if( !a.seq_get_begin( len ) )
    return FALSE;
  s.length( len );
  for( CORBA::ULong i = 0; i < len; i++ )
    if( !(a >>= s[ i ]) )
      return FALSE;
  return a.seq_get_end();
}


CORBA::Boolean
operator<<=( CORBA::Any &a, const SequenceTmpl<CORBA::Long> &s )
{
  static CORBA::TypeCode_ptr tc = NULL;

  if( tc == NULL )
    tc=CORBA::TypeCode::create_sequence_tc(0,CORBA::_tc_long)->mk_constant();
  a.type( tc );
  if( !a.seq_put_begin( s.length() ) )
    return FALSE;
  for( CORBA::ULong i = 0; i < s.length(); i++ )
    if( !(a <<= ((SequenceTmpl<CORBA::Long>&)s)[ i ]) )
      return FALSE;
  return a.seq_put_end();
}

CORBA::Boolean
operator>>=( const CORBA::Any &a, SequenceTmpl<CORBA::Long> &s )
{
  CORBA::ULong len;

  if( !a.seq_get_begin( len ) )
    return FALSE;
  s.length( len );
  for( CORBA::ULong i = 0; i < len; i++ )
    if( !(a >>= s[ i ]) )
      return FALSE;
  return a.seq_get_end();
}


CORBA::Boolean
operator<<=( CORBA::Any &_a, const SequenceTmpl<CORBA::LongLong> &_s )
{
  static CORBA::TypeCode_ptr _tc = NULL;

  if( _tc == NULL )
    _tc=CORBA::TypeCode::create_sequence_tc(
	0, CORBA::_tc_longlong)->mk_constant();
  _a.type( _tc );
  if( !_a.seq_put_begin( _s.length() ) )
    return FALSE;
  for( CORBA::ULong _i = 0; _i < _s.length(); _i++ )
    if( !(_a <<= ((SequenceTmpl<CORBA::LongLong>&)_s)[ _i ]) )
      return FALSE;
  return _a.seq_put_end();
}


CORBA::Boolean
operator>>=( const CORBA::Any &_a, SequenceTmpl<CORBA::LongLong> &_s )
{
  CORBA::ULong _len;

  if( !_a.seq_get_begin( _len ) )
    return FALSE;
  _s.length( _len );
  for( CORBA::ULong _i = 0; _i < _len; _i++ )
    if( !(_a >>= _s[ _i ]) )
      return FALSE;
  return _a.seq_get_end();
}


CORBA::Boolean
operator<<=( CORBA::Any &_a, const SequenceTmpl<CORBA::ULongLong> &_s )
{
  static CORBA::TypeCode_ptr _tc = NULL;

  if( _tc == NULL )
    _tc=CORBA::TypeCode::create_sequence_tc(
	0, CORBA::_tc_ulonglong)->mk_constant();
  _a.type( _tc );
  if( !_a.seq_put_begin( _s.length() ) )
    return FALSE;
  for( CORBA::ULong _i = 0; _i < _s.length(); _i++ )
    if( !(_a <<= ((SequenceTmpl<CORBA::ULongLong>&)_s)[ _i ]) )
      return FALSE;
  return _a.seq_put_end();
}


CORBA::Boolean
operator>>=( const CORBA::Any &_a, SequenceTmpl<CORBA::ULongLong> &_s )
{
  CORBA::ULong _len;

  if( !_a.seq_get_begin( _len ) )
    return FALSE;
  _s.length( _len );
  for( CORBA::ULong _i = 0; _i < _len; _i++ )
    if( !(_a >>= _s[ _i ]) )
      return FALSE;
  return _a.seq_get_end();
}


CORBA::Boolean
operator<<=( CORBA::Any &a, const SequenceTmpl<CORBA::Float> &s )
{
  static CORBA::TypeCode_ptr tc = NULL;

  if( tc == NULL )
    tc=CORBA::TypeCode::create_sequence_tc(0,CORBA::_tc_float)->mk_constant();
  a.type( tc );
  if( !a.seq_put_begin( s.length() ) )
    return FALSE;
  for( CORBA::ULong i = 0; i < s.length(); i++ )
    if( !(a <<= ((SequenceTmpl<CORBA::Float>&)s)[ i ]) )
      return FALSE;
  return a.seq_put_end();
}

CORBA::Boolean
operator>>=( const CORBA::Any &a, SequenceTmpl<CORBA::Float> &s )
{
  CORBA::ULong len;

  if( !a.seq_get_begin( len ) )
    return FALSE;
  s.length( len );
  for( CORBA::ULong i = 0; i < len; i++ )
    if( !(a >>= s[ i ]) )
      return FALSE;
  return a.seq_get_end();
}


CORBA::Boolean
operator<<=( CORBA::Any &a, const SequenceTmpl<CORBA::Double> &s )
{
  static CORBA::TypeCode_ptr tc = NULL;

  if( tc == NULL )
    tc=CORBA::TypeCode::create_sequence_tc(0,CORBA::_tc_double)->mk_constant();
  a.type( tc );
  if( !a.seq_put_begin( s.length() ) )
    return FALSE;
  for( CORBA::ULong i = 0; i < s.length(); i++ )
    if( !(a <<= ((SequenceTmpl<CORBA::Double>&)s)[ i ]) )
      return FALSE;
  return a.seq_put_end();
}

CORBA::Boolean
operator>>=( const CORBA::Any &a, SequenceTmpl<CORBA::Double> &s )
{
  CORBA::ULong len;

  if( !a.seq_get_begin( len ) )
    return FALSE;
  s.length( len );
  for( CORBA::ULong i = 0; i < len; i++ )
    if( !(a >>= s[ i ]) )
      return FALSE;
  return a.seq_get_end();
}


CORBA::Boolean
operator<<=( CORBA::Any &_a, const SequenceTmpl<CORBA::LongDouble> &_s )
{
  static CORBA::TypeCode_ptr _tc = NULL;

  if( _tc == NULL )
    _tc=CORBA::TypeCode::create_sequence_tc(
	0, CORBA::_tc_longdouble)->mk_constant();
  _a.type( _tc );
  if( !_a.seq_put_begin( _s.length() ) )
    return FALSE;
  for( CORBA::ULong _i = 0; _i < _s.length(); _i++ )
    if( !(_a <<= ((SequenceTmpl<CORBA::LongDouble>&)_s)[ _i ]) )
      return FALSE;
  return _a.seq_put_end();
}


CORBA::Boolean
operator>>=( const CORBA::Any &_a, SequenceTmpl<CORBA::LongDouble> &_s )
{
  CORBA::ULong _len;

  if( !_a.seq_get_begin( _len ) )
    return FALSE;
  _s.length( _len );
  for( CORBA::ULong _i = 0; _i < _len; _i++ )
    if( !(_a >>= _s[ _i ]) )
      return FALSE;
  return _a.seq_get_end();
}

CORBA::Boolean
operator<<=( CORBA::Any &_a, const SequenceTmpl<CORBA::WCharWrapper> &_s )
{
  static CORBA::TypeCode_ptr _tc = NULL;

  if( _tc == NULL )
    _tc=CORBA::TypeCode::create_sequence_tc(0,CORBA::_tc_wchar)->mk_constant();
  _a.type( _tc );
  if( !_a.seq_put_begin( _s.length() ) )
    return FALSE;
  for( CORBA::ULong _i = 0; _i < _s.length(); _i++ )
    if( !(_a <<= CORBA::Any::from_wchar( ((SequenceTmpl<CORBA::WCharWrapper>&)_s)[ _i ] )) )
      return FALSE;
  return _a.seq_put_end();
}


CORBA::Boolean
operator>>=( const CORBA::Any &_a, SequenceTmpl<CORBA::WCharWrapper> &_s )
{
  CORBA::ULong _len;

  if( !_a.seq_get_begin( _len ) )
    return FALSE;
  _s.length( _len );
  for( CORBA::ULong _i = 0; _i < _len; _i++ )
    if( !(_a >>= CORBA::Any::to_wchar( _s[ _i ] )) )
      return FALSE;
  return _a.seq_get_end();
}

CORBA::Boolean
operator<<=( CORBA::Any &_a, const SequenceTmpl<CORBA::WString_var> &_s )
{
  static CORBA::TypeCode_ptr _tc = NULL;

  if( _tc == NULL )
    _tc=CORBA::TypeCode::create_sequence_tc(
	0, CORBA::_tc_wstring)->mk_constant();
  _a.type( _tc );
  if( !_a.seq_put_begin( _s.length() ) )
    return FALSE;
  for( CORBA::ULong _i = 0; _i < _s.length(); _i++ )
    if( !(_a <<= ((SequenceTmpl<CORBA::WString_var>&)_s)[ _i ]) )
      return FALSE;
  return _a.seq_put_end();
}

CORBA::Boolean
operator>>=( const CORBA::Any &_a, SequenceTmpl<CORBA::WString_var> &_s )
{
  CORBA::ULong _len;

  if( !_a.seq_get_begin( _len ) )
    return FALSE;
  _s.length( _len );
  for( CORBA::ULong _i = 0; _i < _len; _i++ )
    if( !(_a >>= _s[ _i ]) )
      return FALSE;
  return _a.seq_get_end();
}


CORBA::Boolean
operator<<=( CORBA::Any &a, const SequenceTmpl<CORBA::String_var> &s )
{
  static CORBA::TypeCode_ptr tc = NULL;

  if( tc == NULL )
    tc=CORBA::TypeCode::create_sequence_tc(0,CORBA::_tc_string)->mk_constant();
  a.type( tc );
  if( !a.seq_put_begin( s.length() ) )
    return FALSE;
  for( CORBA::ULong i = 0; i < s.length(); i++ )
    if( !(a <<= ((SequenceTmpl<CORBA::String_var>&)s)[ i ]) )
      return FALSE;
  return a.seq_put_end();
}

CORBA::Boolean
operator>>=( const CORBA::Any &a, SequenceTmpl<CORBA::String_var> &s )
{
  CORBA::ULong len;

  if( !a.seq_get_begin( len ) )
    return FALSE;
  s.length( len );
  for( CORBA::ULong i = 0; i < len; i++ )
    if( !(a >>= s[ i ]) )
      return FALSE;
  return a.seq_get_end();
}

CORBA::Boolean
operator<<=( CORBA::Any &_a,
	     const SequenceTmpl<CORBA::Object_var> &_s )
{
  static CORBA::TypeCode_ptr _tc = NULL;

  if( _tc == NULL )
    _tc=CORBA::TypeCode::create_sequence_tc(
	0,CORBA::_tc_Object)->mk_constant();
  _a.type( _tc );
  if( !_a.seq_put_begin( _s.length() ) )
    return FALSE;
  for( CORBA::ULong _i = 0; _i < _s.length(); _i++ )
    if( !(_a <<= CORBA::Any::from_object( 
                    ((SequenceTmpl<CORBA::Object_var>&)_s)[ _i ], "Object" )) )
      return FALSE;
  return _a.seq_put_end();
}

CORBA::Boolean
operator>>=( const CORBA::Any &_a,
	     SequenceTmpl<CORBA::Object_var> &_s )
{
  CORBA::ULong _len;

  if( !_a.seq_get_begin( _len ) )
    return FALSE;
  _s.length( _len );
  for( CORBA::ULong _i = 0; _i < _len; _i++ )
    if( !(_a >>= CORBA::Any::to_object( _s[ _i ] )) )
      return FALSE;
  return _a.seq_get_end();
}
