/*
 *  MICO --- a free CORBA implementation
 *  Copyright (C) 1997-98 Kay Roemer & Arno Puder
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Send comments and/or bug reports to:
 *                 mico@informatik.uni-frankfurt.de
 */

#include <CORBA.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <algorithm>
#include <mico/impl.h>
#include <mico/template_impl.h>
#include <mico/util.h>
#include <mico/os-net.h>


#ifdef _DEBUG
OSNet::WSA_Number_Text_struct
OSNet::WSA_Number_Text_table[WSA_Number_Text_table_n] = {
    { ENAMETOOLONG, "ENAMETOOLONG" },
    { ENOTEMPTY, "ENOTEMPTY" },
    { EWOULDBLOCK, "EWOULDBLOCK" },
    { HOST_NOT_FOUND, "HOST_NOT_FOUND" },
    { NO_ADDRESS, "NO_ADDRESS" },
    { NO_DATA, "NO_DATA" },
    { NO_RECOVERY, "NO_RECOVERY" },
    { TRY_AGAIN, "TRY_AGAIN" },
    { WSABASEERR, "WSABASEERR" },
    { WSAEACCES, "WSAEACCES" },
    { WSAEADDRINUSE, "WSAEADDRINUSE" },
    { WSAEADDRNOTAVAIL, "WSAEADDRNOTAVAIL" },
    { WSAEAFNOSUPPORT, "WSAEAFNOSUPPORT" },
    { WSAEALREADY, "WSAEALREADY" },
    { WSAEBADF, "WSAEBADF" },
    { WSAECANCELLED, "WSAECANCELLED" },
    { WSAECONNABORTED, "WSAECONNABORTED" },
    { WSAECONNREFUSED, "WSAECONNREFUSED" },
    { WSAECONNRESET, "WSAECONNRESET" },
    { WSAEDESTADDRREQ, "WSAEDESTADDRREQ" },
    { WSAEDISCON, "WSAEDISCON" },
    { WSAEDQUOT, "WSAEDQUOT" },
    { WSAEFAULT, "WSAEFAULT" },
    { WSAEHOSTDOWN, "WSAEHOSTDOWN" },
    { WSAEHOSTUNREACH, "WSAEHOSTUNREACH" },
    { WSAEINPROGRESS, "WSAEINPROGRESS" },
    { WSAEINTR, "WSAEINTR" },
    { WSAEINVAL, "WSAEINVAL" },
    { WSAEINVALIDPROCTABLE, "WSAEINVALIDPROCTABLE" },
    { WSAEINVALIDPROVIDER, "WSAEINVALIDPROVIDER" },
    { WSAEISCONN, "WSAEISCONN" },
    { WSAELOOP, "WSAELOOP" },
    { WSAEMFILE, "WSAEMFILE" },
    { WSAEMSGSIZE, "WSAEMSGSIZE" },
    { WSAENAMETOOLONG, "WSAENAMETOOLONG" },
    { WSAENETDOWN, "WSAENETDOWN" },
    { WSAENETRESET, "WSAENETRESET" },
    { WSAENETUNREACH, "WSAENETUNREACH" },
    { WSAENOBUFS, "WSAENOBUFS" },
    { WSAENOMORE, "WSAENOMORE" },
    { WSAENOPROTOOPT, "WSAENOPROTOOPT" },
    { WSAENOTCONN, "WSAENOTCONN" },
    { WSAENOTEMPTY, "WSAENOTEMPTY" },
    { WSAENOTSOCK, "WSAENOTSOCK" },
    { WSAEOPNOTSUPP, "WSAEOPNOTSUPP" },
    { WSAEPFNOSUPPORT, "WSAEPFNOSUPPORT" },
    { WSAEPROCLIM, "WSAEPROCLIM" },
    { WSAEPROTONOSUPPORT, "WSAEPROTONOSUPPORT" },
    { WSAEPROTOTYPE, "WSAEPROTOTYPE" },
    { WSAEPROVIDERFAILEDINIT, "WSAEPROVIDERFAILEDINIT" },
    { WSAEREFUSED, "WSAEREFUSED" },
    { WSAEREMOTE, "WSAEREMOTE" },
    { WSAESHUTDOWN, "WSAESHUTDOWN" },
    { WSAESOCKTNOSUPPORT, "WSAESOCKTNOSUPPORT" },
    { WSAESTALE, "WSAESTALE" },
    { WSAETIMEDOUT, "WSAETIMEDOUT" },
    { WSAETOOMANYREFS, "WSAETOOMANYREFS" },
    { WSAEUSERS, "WSAEUSERS" },
    { WSAEWOULDBLOCK, "WSAEWOULDBLOCK" },
    { WSAHOST_NOT_FOUND, "WSAHOST_NOT_FOUND" },
    { WSANOTINITIALISED, "WSANOTINITIALISED" },
    { WSANO_DATA, "WSANO_DATA" },
    { WSANO_RECOVERY, "WSANO_RECOVERY" },
    { WSASERVICE_NOT_FOUND, "WSASERVICE_NOT_FOUND" },
    { WSASYSCALLFAILURE, "WSASYSCALLFAILURE" },
    { WSASYSNOTREADY, "WSASYSNOTREADY" },
    { WSATRY_AGAIN, "WSATRY_AGAIN" },
    { WSATYPE_NOT_FOUND, "WSATYPE_NOT_FOUND" },
    { WSAVERNOTSUPPORTED, "WSAVERNOTSUPPORTED" },
    { WSA_E_CANCELLED, "WSA_E_CANCELLED" },
    { WSA_E_NO_MORE, "WSA_E_NO_MORE" },
    { WSA_FLAG_MULTIPOINT_C_LEAF, "WSA_FLAG_MULTIPOINT_C_LEAF" },
    { WSA_FLAG_MULTIPOINT_C_ROOT, "WSA_FLAG_MULTIPOINT_C_ROOT" },
    { WSA_FLAG_MULTIPOINT_D_LEAF, "WSA_FLAG_MULTIPOINT_D_LEAF" },
    { WSA_FLAG_MULTIPOINT_D_ROOT, "WSA_FLAG_MULTIPOINT_D_ROOT" },
    { WSA_FLAG_OVERLAPPED, "WSA_FLAG_OVERLAPPED" },
    { WSA_INFINITE, "WSA_INFINITE" },
    { WSA_INVALID_HANDLE, "WSA_INVALID_HANDLE" },
    { WSA_INVALID_PARAMETER, "WSA_INVALID_PARAMETER" },
    { WSA_IO_INCOMPLETE, "WSA_IO_INCOMPLETE" },
    { WSA_IO_PENDING, "WSA_IO_PENDING" },
    { WSA_MAXIMUM_WAIT_EVENTS, "WSA_MAXIMUM_WAIT_EVENTS" },
    { WSA_NOT_ENOUGH_MEMORY, "WSA_NOT_ENOUGH_MEMORY" },
    { WSA_OPERATION_ABORTED, "WSA_OPERATION_ABORTED" },
    { WSA_WAIT_EVENT_0, "WSA_WAIT_EVENT_0" },
    { WSA_WAIT_FAILED, "WSA_WAIT_FAILED" },
    { WSA_WAIT_IO_COMPLETION, "WSA_WAIT_IO_COMPLETION" },
    { WSA_WAIT_TIMEOUT, "WSA_WAIT_TIMEOUT" }
};
#endif


/**************************** UnixProcess *****************************/


MICO::UnixProcess::ListProcess MICO::UnixProcess::_procs;

MICO::UnixProcess::UnixProcess (const char *cmd, MICO::ProcessCallback *cb)
{
    _exit_status = -1;
    _pid = 0;
    _detached = FALSE;
    _cb = cb;
    _procs.push_back (this);
    _args = cmd;
}

MICO::UnixProcess::~UnixProcess ()
{
    ListProcess::iterator pos = find (_procs.begin(), _procs.end(), this);
    assert (pos != _procs.end());
    _procs.erase (pos);

    if (!_detached && !exited())
	terminate ();
}

CORBA::Boolean
MICO::UnixProcess::run ()
{
    assert (0);
    return FALSE;
}

CORBA::Boolean
MICO::UnixProcess::exited ()
{
    return _pid <= 0 || _exit_status >= 0;
}

CORBA::Boolean
MICO::UnixProcess::exit_status ()
{
    return _exit_status == 0;
}

void
MICO::UnixProcess::terminate ()
{
    assert (0);
}

void
MICO::UnixProcess::detach ()
{
    _detached = TRUE;
}

MICO::UnixProcess::operator CORBA::Boolean ()
{
    return _pid > 0;
}


/*************************** UnixSharedLib ****************************/


MICO::UnixSharedLib::UnixSharedLib (const char *name)
{
    _name = name;
    _handle = 0;
}

MICO::UnixSharedLib::~UnixSharedLib ()
{
}

void *
MICO::UnixSharedLib::symbol (const char *sym)
{
    return 0;
}

const char *
MICO::UnixSharedLib::error ()
{
    return "no shlib support";
}

MICO::UnixSharedLib::operator CORBA::Boolean ()
{
    return FALSE;
}

const char *
MICO::UnixSharedLib::name ()
{
    return _name.c_str();
}
